% Plot differential sensitivity vs. R_w
% Uses spectral_traces.mat.

clear all
%close all

select_waveforms = 2;

switch select_waveforms
    case 1
        waveform_dir = 'wfm_preclinical';
    case 2
        waveform_dir = 'wfm_qMAS';
end

spectral_traces_path = fullfile(pwd,waveform_dir,'spectral_traces.mat');


load(spectral_traces_path)

% ----------- select waveforms to compare

switch select_waveforms
    case 1
        wfm_ind(1,:) = find(contains({wfms_trace_data.name},'LTE_3')); % detuned
        wfm_ind(2,:) = find(contains({wfms_trace_data.name},'LTE_1')); % tuned
        wfm_ind(3,:) = find(contains({wfms_trace_data.name},'STE')); % STE
        
        figNum = 1;
        
    case 2
        wfm_ind(1,:) = find(contains({wfms_trace_data.name},'qMAS_detunedLTE'));
        wfm_ind(2,:) = find(contains({wfms_trace_data.name},'qMAS_tunedLTE'));
        wfm_ind(3,:) = find(contains({wfms_trace_data.name},'qMAS_SE'));
        
        figNum = 2;
end

N = size(wfm_ind,2);
D0 = 2.5e-9; %2.4800e-09;


for M = 1:N
    cumtrace1 = wfms_trace_data(wfm_ind(1,M)).cumTrace; % detuned
    f1 = wfms_trace_data(wfm_ind(1,M)).f;
    cumtrace2 = wfms_trace_data(wfm_ind(2,M)).cumTrace; % tuned
    f2 = wfms_trace_data(wfm_ind(2,M)).f;
    cumtrace3 = wfms_trace_data(wfm_ind(3,M)).cumTrace; % STE
    f3 = wfms_trace_data(wfm_ind(3,M)).f;
    
    L1 = length(cumtrace1);
    L2 = length(cumtrace2);
    L3 = length(cumtrace3);
    
    L = min(L1,L2); % tuned - detuned
    tmp1 = cumtrace1(1:L); tmp2 = cumtrace2(1:L); f = f1(1:L);
    toPlot(M).DetunedTuned = abs(tmp1-tmp2);
    toPlot(M).DetunedTuned_f = f;
    
    L = min(L1,L3); % Detuned - STE
    tmp1 = cumtrace1(1:L); tmp2 = cumtrace3(1:L); f = f1(1:L);
    toPlot(M).DetunedSTE = abs(tmp1-tmp2);
    toPlot(M).DetunedSTE_f = f;
    
    L = min(L2,L3); % tuned - STE
    tmp1 = cumtrace2(1:L); tmp2 = cumtrace3(1:L); f = f2(1:L);
    toPlot(M).TunedSTE = abs(tmp1-tmp2);
    toPlot(M).TunedSTE_f = f;
    
end

%------ plot -----
XLIM = [0 8];
YLIM = [0 .6];
showLables = 0;
col = [1 0 0 ; ...
    0 .8 0;...
    0 0 1;...
    0 0 0];

lw = 3;
fig.H = 700; %800;
fig.W = 400;
fig.fs = 16;
fig.lw = 3;

fh = figure(figNum);
clf
fh.Color = [1 1 1];
pause(.1)
fh.Position = [800 200 fig.W fig.H];
subFigH = .26;
subFigW = .78;
subFigLeft = .15;
subFigBottom = .05+1.25*subFigH*[2 1 0];

axes('position',[subFigLeft subFigBottom(1) subFigW subFigH]);
hold on
for M = 1:N
    X = toPlot(M).DetunedSTE_f;
    X = sqrt(D0./(2*pi*X))*1e6;
    Y = toPlot(M).DetunedSTE;
    plot(X,Y,'LineWidth',lw,'Color',col(M,:))
end
if showLables
    ylabel('Detuned - STE')
end
set(gca, 'XScale', 'lin', 'YScale', 'lin',...
    'FontSize',fig.fs,'box','off','TickDir','out','LineWidth',fig.lw,'TickLength',[.02 .02])
set(gca, 'XScale', 'lin', 'YScale', 'lin',...
    'FontSize',fig.fs,'box','off','TickDir','out','LineWidth',fig.lw,'TickLength',[.02 .02])
xlim(XLIM)
ylim(YLIM)

axes('position',[subFigLeft subFigBottom(2) subFigW subFigH]);
hold on
for M = 1:N
    X = toPlot(M).TunedSTE_f;
    X = sqrt(D0./(2*pi*X))*1e6;
    Y = toPlot(M).TunedSTE;
    plot(X,Y,'LineWidth',lw,'Color',col(M,:))
end
if showLables
    ylabel('Tuned - STE')
end
set(gca, 'XScale', 'lin', 'YScale', 'lin',...
    'FontSize',fig.fs,'box','off','TickDir','out','LineWidth',fig.lw,'TickLength',[.02 .02])
set(gca, 'XScale', 'lin', 'YScale', 'lin',...
    'FontSize',fig.fs,'box','off','TickDir','out','LineWidth',fig.lw,'TickLength',[.02 .02])
xlim(XLIM)
ylim(YLIM)

axes('position',[subFigLeft subFigBottom(3) subFigW subFigH]);
hold on
for M = 1:N
    X = toPlot(M).DetunedTuned_f;
    X = sqrt(D0./(2*pi*X))*1e6;
    Y = toPlot(M).DetunedTuned;
    plot(X,Y,'LineWidth',lw,'Color',col(M,:))
end
if showLables
    ylabel('Detuned - Tuned')
    xlabel('R_{w} [\mu m]')
end
set(gca, 'XScale', 'lin', 'YScale', 'lin',...
    'FontSize',fig.fs,'box','off','TickDir','out','LineWidth',fig.lw,'TickLength',[.02 .02])
xlim(XLIM)
ylim(YLIM)






