% Extract spectral traces (saved in the waveform folders) - waveforms need to be analyzed first!
% Used by make_lookup_table_DvsR_mouse.m and differential_sensitivity.m

clear all

select_waveforms = 1;

switch select_waveforms
    case 1
        waveform_dir = 'wfm_preclinical';
        waveform_names = {'M0_LTE_1','M0_LTE_2', 'M0_LTE_3', 'M0_STE', ...
            'M1_LTE_1','M1_LTE_2','M1_LTE_3','M1_STE',...
            'M2_LTE_1','M2_LTE_2','M2_LTE_3','M2_STE',...
            'PGSE_LTE'};
    case 2
        waveform_dir = 'wfm_qMAS';
        waveform_names = {'qMAS_detunedLTE_SE_flip1', 'qMAS_tunedLTE_SE_flip1', 'qMAS_SE_flip1'};
end

waveform_dir = fullfile(pwd,waveform_dir);
saveData = 1;

% spectral traces
trPS_thresh = 0.999;
for n = 1:numel(waveform_names)
    display(n)
    path = fullfile(waveform_dir,[waveform_names{n} '_info.mat']);
    load(path)
    
    wfms_trace_data(n).trPS_thresh = trPS_thresh;
    
    wfms_trace_data(n).f0 = wfm.f(max(find(wfm.trPS<trPS_thresh)));
    wfms_trace_data(n).f_max = max(wfm.f);
    
    ind = find(abs(wfm.f_full)<wfms_trace_data(n).f0);
    wfms_trace_data(n).f_full = wfm.f_full(ind);
    PS_full = wfm.PS_full(ind,:,:);
    tmp_trace = PS_full(:,1,1)+PS_full(:,2,2)+PS_full(:,3,3);
    b = sum(tmp_trace);
    wfms_trace_data(n).trPS_full = tmp_trace/b; % normalize
    %figure(1),clf,plot(wfms(n).f_full,tmp_trace)
    %         wfms(n).PS_full = PS_full/b;% normalize
    
    ind = find(wfm.f<wfms_trace_data(n).f0);
    wfms_trace_data(n).f = wfm.f(ind);
    PS = wfm.PS(ind,:,:);
    tmp_trace = PS(:,1,1)+PS(:,2,2)+PS(:,3,3);
    b = sum(tmp_trace);
    wfms_trace_data(n).trPS = tmp_trace/b; % normalize
    %         figure(1),clf,plot(wfms(n).f,wfms(n).trPS/2)
    %         hold on, plot(wfms(n).f_full,wfms(n).trPS_full,'--')
    wfms_trace_data(n).Vw_norm = (2*pi)^2*wfms_trace_data(n).f.^2*wfms_trace_data(n).trPS;
    
    % cumulative trace
    wfms_trace_data(n).cumTrace = cumsum(wfms_trace_data(n).trPS);
    ind = max(find(wfms_trace_data(n).cumTrace<1/3));
    wfms_trace_data(n).w2_thresh = (2*pi)^2*wfms_trace_data(n).f(ind).^2;
    
    
    dt = wfm.TE/length(wfm.q);
    q = wfm.q/sqrt(wfm.b);
    
    q = sqrt(q(:,1).^2+q(:,2).^2+q(:,3).^2);
    wfms_trace_data(n).maxq_atb1 = max(q);
    
    g = wfm.g/sqrt(wfm.b);
    g = sqrt(g(:,1).^2+g(:,2).^2+g(:,3).^2);
    wfms_trace_data(n).maxg_atb1 = max(g);
    
    
    wfms_trace_data(n).df = wfm.f(2)-wfm.f(1);
    wfms_trace_data(n).b_delta = wfm.b_delta;
    wfms_trace_data(n).TE = wfm.TE;
    wfms_trace_data(n).name = waveform_names{n};
    
end

if saveData
    save(fullfile(waveform_dir,'spectral_traces'),'wfms_trace_data');
end