% Fit and plot mouse heart data with the hindered-restricted model using lookup table.
% Saves fitting results in the *data* subfolders.

clear all

do_MD = 1;
do_Sig = 1;

do_MD_fit = 0; %if 0 then load saved
weightedFit = 1; % weight residuals based on error in experimental MD estimation
saveFig = 0;

select_heart = 1; % 1 or 2

%close all
nFig = select_heart;



% -------------------- select ROI  ----------

data_dir = fullfile(pwd,'data');

wfm_names = {'PGSE_LTE', ...
    'M0_LTE_1', 'M0_LTE_2', 'M0_LTE_3', 'M0_STE', ...
    'M1_LTE_1', 'M1_LTE_2', 'M1_LTE_3', 'M1_STE',...
    'M2_LTE_1', 'M2_LTE_2', 'M2_LTE_3', 'M2_STE'};

baseName = 'lookup_table_mouse';

switch select_heart
    case 1
        
        data_subdir = fullfile(pwd,'data/mouse/heart1');
        
        m = [0 0 0 0 0 1 1 1 1 2 2 2 2];
        channel = [0 1 2 3 4 1 2 3 4 1 2 3 4]/4;
        
        data_files = {'md_41', 'md_42', 'md_43', 'md_44', 'md_45', 'md_47', 'md_48', ...
            'md_49', 'md_50', 'md_52', 'md_53', 'md_54', 'md_55'};
        
        ind_select = 1:numel(wfm_names);
        remove_wfm = {'M0_LTE_2'};
        %remove_wfm = {'none'};
        ind_select = find(~strcmp(wfm_names,remove_wfm(1)));
        
    case 2
        
        data_subdir = fullfile(pwd,'data/mouse/heart2');
        
        m = [0 0 0 0 0 1 1 1 1 2 2 2 2];
        channel = [0 1 2 3 4 1 2 3 4 1 2 3 4]/4;
        
        data_files = {'md_37', 'md_38', 'md_80', 'md_40', 'md_41', 'md_43', 'md_44', ...
            'md_45', 'md_46', 'md_48', 'md_49', 'md_50', 'md_51'};
        ind_select = 1:numel(wfm_names);
        
end

ind_high_b = numel(wfm_names)-3:numel(wfm_names);


%------------ adjust colors
c = zeros(length(m),3);
c(find(m==0),1) = 1+c(find(m==0),1);
c(find(m==1),2) = 1+c(find(m==1),2);
c(find(m==2),3) = 1+c(find(m==2),3);

for n = 1:length(channel)
    c(n,:) = channel(n).^(0.6)*c(n,:);
end


switch 1 %1-cylinder, 2-stick, 3-ellipsoid, 4-sphere
    case 1
        auxP.shape = 2; LookupTableName = [baseName '_cylinder'];
    case 2
        auxP.shape = 4; LookupTableName = [baseName '_stick'];
    case 3
        auxP.shape = 3; LookupTableName = [baseName '_ellipsoid'];
    case 4
        auxP.shape = 1; LookupTableName = [baseName '_sphere'];
end

% read lookup table
load(fullfile(data_dir,[LookupTableName '.mat']),'R')

wfm_names_selected = wfm_names(ind_select);
b_delta_selected = R.b_delta(ind_select);
m_selected = m(ind_select);
c_selected = c(ind_select,:);

MD_calc_selected = R.MD(ind_select,:);
VA_calc_selected = R.VA(ind_select,:);

fs = 18;
lw = 3;


try
    % read experimental MD
    for n = 1:numel(data_files)
        mat_file = [data_files{n} '.mat'];
        load(fullfile(data_subdir,mat_file))
        
        MD_exp.mean(n) = md.roi_mean;
        MD_exp.std(n) = md.roi_sd;
        MD_exp.name{n} = data_files{n};
    end
end


try
    MD_exp_selected = MD_exp.mean(ind_select)*1e-9;
    STD_exp_selected = MD_exp.std(ind_select)*1e-9;
end


%--------------------- fit ---------------------

if do_MD_fit
    
    Nfit = 500; %500
    fh = linspace(0,1,Nfit);
    Dh = linspace(1e-11,2.5e-9,Nfit);
    %Dh = linspace(1e-12,2.5e-9,Nfit);
    
    tic
    if weightedFit
        [nR, fh, Dh, RMSE] = fFit2weighted(MD_calc_selected, MD_exp_selected, STD_exp_selected, fh, Dh);
        
    else
        [nR, fh, Dh, RMSE] = fFit2(MD_calc_selected, MD_exp_selected, fh, Dh);
        
    end
    toc
    
    fitRes.Nfit = Nfit;
    fitRes.nR = nR;
    fitRes.R = R.Rs(nR);
    fitRes.fh = fh;
    fitRes.Dh = Dh;
    fitRes.RMSE = RMSE;
    
    fitRes.data_name = R.name;
    fitRes.calc_name = LookupTableName;
    
    fitRes.D0 = R.D0;
    fitRes.auxP = R.auxP;
    fitRes.b_delta = R.b_delta;
    fitRes.TE = R.TE;
    fitRes.ind_select = ind_select;
    
    mkdir(fullfile(data_subdir,'fit'))
    if weightedFit
        save(fullfile(data_subdir,'fit',[LookupTableName '_fit_weighted.mat']),'fitRes')
    else
        save(fullfile(data_subdir,'fit',[LookupTableName '_fit.mat']),'fitRes')
    end
    
    
else
    
    try
        if weightedFit
            load(fullfile(data_subdir,'fit',[LookupTableName '_fit_weighted.mat']),'fitRes')
        else
            load(fullfile(data_subdir,'fit',[LookupTableName '_fit.mat']),'fitRes')
        end
    end
    
end

try
    Y = fitRes.fh*fitRes.Dh + (1-fitRes.fh)*MD_calc_selected;  % calculated
catch
    Y = (1-fr)*Dh + fr*MD_calc_selected;  % calculated
end
MD_calc_2compartments = Y;


% ---- plot MD ------
fh = figure(1+10*nFig);,clf
hold on
for n = 1:size(Y,1)
    if b_delta_selected(n)
        LineStyle = '--';
    else
        LineStyle = '-';
    end
    %         if m_selected(n) == 100
    %             c(n,1) = c(n,1);%*0.99;
    %             c(n,2) = c(n,2);%*0.99;
    %             c(n,3) = 1; %c(n,3)*0.7;
    %         end
    plot(R.Rs*1e6,Y(n,:)*1e9,LineStyle,'Color',c_selected(n,:),'LineWidth',3)
end

try
    for n = 1:length(MD_exp_selected)
        if b_delta_selected(n) == 1
            MarkerStyle = 'x'; MarkerSize = 12;
        else
            MarkerStyle = 'o'; MarkerSize = 9;
        end
        
        plot(R.Rs(fitRes.nR)*1e6,MD_exp_selected(n)*1e9,MarkerStyle,'Color',c_selected(n,:),'MarkerSize',MarkerSize,'LineWidth',2)
        %plot(R.Rs(fitRes.nR)*1e6,MD_exp_selected(n)*1e9,'.','Color',1*c(n,:),'MarkerSize',MarkerSize,'LineWidth',2)
        
        %         errorbar(R.Rs(nR)*1e6,MD_exp_selected(n)*1e9,0.05*STD_exp_selected(n)*1e9,MarkerStyle,'Color',c(n,:),'MarkerSize',MarkerSize,'LineWidth',2)
        
    end
end
ylim([0.99*min(Y(:)) 1.01*max(Y(:))]*1e9)
%pbaspect([1 1.61803 1])
pbaspect([1 1 1])
XTicks = [1 2 5 10 20 30 40 50];
set(gca,'XTick',XTicks)
set(gca, 'XScale', 'log','YScale', 'lin',...
    'FontSize',fs,'box','off','TickDir','out','LineWidth',lw,'TickLength',[.02 .02])


%lh = legend(wfm_names_selected,'Interpreter', 'none','box','off','location','best');
wfm_names_legend = {};
for n = 1:length(wfm_names_selected)
    wfm_names_legend{end+1} = ' ';
end
%     lh = legend(wfm_names_legend,'Interpreter', 'none','box','off','location','best');
%     lh.ItemTokenSize(1) = lh.ItemTokenSize(1)*1.3;

try
    title_str = sprintf('%s: R = %.2f um, fr = %.2f, Dh = %.2f, RMSE = %.4g', LookupTableName,fitRes.R*1e6, 1-fitRes.fh, fitRes.Dh*1e9, fitRes.RMSE);
    title(title_str, 'Interpreter', 'none','FontSize',10)
    
    if saveFig
        if weightedFit
            print(fh,fullfile(data_subdir,'fit',[LookupTableName '_fit_weighted.png']),'-dpng','-r300');
        else
            print(fh,fullfile(data_subdir,'fit',[LookupTableName '_fit.png']),'-dpng','-r300');
        end
    end
catch
    do_MD = 0;
end


% correlation - errors
if (1)
    fh = figure(2+1+10*nFig);,clf
    hold on
    try
        for n = 1:length(MD_exp_selected)
            Error = (MD_exp_selected(n)-Y(n,fitRes.nR))*1e9;
            Xplot = Y(n,fitRes.nR)*1e9;
            Yplot = MD_exp_selected(n)*1e9;
            
            if b_delta_selected(n) == 1
                MarkerStyle = 'x'; MarkerSize = 12;
            else
                MarkerStyle = 'o'; MarkerSize = 9;
            end
            ph(n)=plot(Xplot,Yplot,MarkerStyle,'Color',c_selected(n,:),'MarkerSize',MarkerSize,'LineWidth',2);
            %plot(Xplot,Yplot,'.','Color',1*c_selected(n,:),'MarkerSize',MarkerSize,'LineWidth',2)
            errorbar(Xplot,Yplot,abs(Error),MarkerStyle,'Color',c_selected(n,:),'MarkerSize',MarkerSize,'LineWidth',2)
        end
        %             Xplot = Y(:,fitRes.nR)*1e9;
        Xplot = [min(Y(:,fitRes.nR)) max(Y(:,fitRes.nR))]*1e9;
        plot(Xplot,Xplot,':k','LineWidth',1);
        
        %             lh = legend(ph,wfm_names_selected,'Interpreter', 'none','box','off','location','best');
        for n = 1:length(wfm_names_selected)
            wfm_names_legend{end+1} = ' ';
        end
        %             lh = legend(ph,wfm_names_legend,'Interpreter', 'none','box','off','location','best');
        %             lh.ItemTokenSize(1) = lh.ItemTokenSize(1)*1.3;
        
    end
    
    Xplot = Y(:,fitRes.nR)*1e9;
    Yplot = MD_exp_selected(:)*1e9;
    
    xlim([0.98*min([Xplot; Yplot]) 1.01*max([Xplot; Yplot])])
    ylim([0.98*min([Xplot; Yplot]) 1.01*max([Xplot; Yplot])])
    
    pbaspect([1 1 1])
    
    set(gca, 'YScale', 'lin',...
        'FontSize',fs,'box','off','TickDir','out','LineWidth',lw,'TickLength',[.02 .02])
    
    
    try
        title_str = sprintf('%s: R = %.2f um, fr = %.2f, Dh = %.2f, RMSE = %.4g', LookupTableName,fitRes.R*1e6, 1-fitRes.fh, fitRes.Dh*1e9, fitRes.RMSE);
        title(title_str, 'Interpreter', 'none','FontSize',10)
        
        if (0) %saveFig
            if weightedFit
                print(fh,fullfile(data_subdir,'fit',[LookupTableName '_fit_weighted.png']),'-dpng','-r300');
            else
                print(fh,fullfile(data_subdir,'fit',[LookupTableName '_fit.png']),'-dpng','-r300');
            end
        end
    catch
        do_MD = 0;
    end
end

display(sprintf('%s: R = %.2f um, fr = %.2f, Dh = %.2f, RMSE = %.4g, D0 = %.4g', ...
    LookupTableName,fitRes.R*1e6, 1-fitRes.fh, fitRes.Dh*1e9, fitRes.RMSE, fitRes.D0));




%--------------------------------- E -signal-  -----------------------------

for n = 1:length(ind_high_b)
    SigName{n} = data_files{ind_high_b(n)};
    mat_file = [SigName{n} '.mat'];
    load(fullfile(data_subdir,mat_file))
    
    s = md.Savg';
    b(:,n) = md.b'*1e6;
    
    % normalize
    p = polyfit(b([1:3],n),-log(s([1:3])),1);
    s0 = exp(-p(2));
    
    Sig_exp(:,n) = s/s0;
end

b_delta_high_b = b_delta_selected(ind_high_b-1);
m_high_b = m_selected(ind_high_b-1);

% signal vs b
fh = figure(3+10*nFig);,clf
hold on
plot(b(:,1)*1e-6, Sig_exp(:,1),'--x','Color',[1 0 0],'MarkerSize',12,'LineWidth',2)
plot(b(:,2)*1e-6, Sig_exp(:,2),'.:','Color',[0 0.8 0],'MarkerSize',30,'LineWidth',2)
plot(b(:,3)*1e-6, Sig_exp(:,3),'--*','Color',[0 0 1],'MarkerSize',12,'LineWidth',2)
plot(b(:,4)*1e-6, Sig_exp(:,4),'-o','Color',[0 0 0],'MarkerSize',10,'LineWidth',2)


set(gca, 'YScale', 'log',...
    'FontSize',fs,'box','off','TickDir','out','LineWidth',lw,'TickLength',[.02 .02])
pbaspect([1.1 1 1])

wfm_names_legend = {};
for n = 1:4
    wfm_names_legend{end+1} = ' ';
end

% MD for M2
col = [1 0 0;...
    0 .8 0;...
    0 0 1;...
    0 0 0];
ms = 24;

fh = figure(4+10*nFig);,clf
hold on
for n = 1:4
    eD = abs(MD_exp.std(9+n));
    D = MD_exp.mean(9+n);
    x = n + .1*(select_heart-1);
    errorbar(x,D,eD,'.','Color',col(n,:),'MarkerSize',ms,'LineWidth',2)
    %plot(n,D,'.','Color',col(n,:),'MarkerSize',18,'LineWidth',2)
end
xticks([1:4])
xticklabels({})
pbaspect([.85 1 1])

set(gca, 'YScale', 'log',...
    'FontSize',fs,'box','off','TickDir','out','LineWidth',lw,'TickLength',[.02 .02])





% ------------- functions ---------------------------------------------

function [nR, fh, Dh, RMSE] = fFit2weighted(MDcalc, MDexp, STDexp, fh, Dh)
NR = size(MDcalc,2);
MDexp_mat = repmat(MDexp',1,NR);
STDexp_mat = repmat(STDexp',1,NR);

for nf = 1:length(fh)
    for nD = 1: length(Dh)
        Dcalc = fh(nf)*Dh(nD) + (1-fh(nf))*MDcalc;
        rMSE(:,nf,nD) = sum((MDexp_mat-Dcalc).^2./STDexp_mat,1);
    end
end

minimum=min(rMSE(:));

%convert linear indices to subscript indices
%[nR,minF,minD]=ind2sub(size(MSE),find(MSE==minimum));
[nR,nf,nD]=ind2sub(size(rMSE),find(rMSE==minimum));


if length(nR)>1
    disp('more than 1 minima');
    nR = nR(1);
    nf = nf(1);
    nD = nD(1);
end

fh = fh(nf);
Dh = Dh(nD);

Dcalc = fh*Dh + (1-fh)*MDcalc(:,nR);
RMSE = sqrt(sum((MDexp_mat(:,nR)-Dcalc).^2/length(MDexp)));


end


function [nR, fh, Dh, RMSE] = fFit(MD, MDexp, fh, Dh, Dratio)
NR = size(MD,1);
MDexp_mat = repmat(MDexp,1,NR)';

for nf = 1:length(fh)
    for nD = 1: length(Dh)
        Dm = fh(nf)*Dh(nD) + (1-fh(nf))*MD*Dratio;
        MSDif(:,nf,nD) = sum((MDexp_mat- Dm).^2,2);
    end
end

minimum=min(MSDif(:));
%convert linear indices to subscript indices
[nR,minF,minD]=ind2sub(size(MSDif),find(MSDif==minimum));

if length(nR)>1
    disp('more than 1 minima');
    nR = nR(1);
    minF = minF(1);
    minD = minD(1);
end

fh = fh(minF);
Dh = Dh(minD);

RMSE = sqrt(MSDif(nR,minF,minD)/length(MDexp));

end


