% Fit and plot pig heart data with the hindered-restricted model using lookup table.
% Saves fitting results in the *data* subfolders.

clear all
close all

restoredefaultpath
addpath(genpath(fullfile(pwd,'add')));

data_dir = fullfile(pwd,'data');
data_subdir = fullfile(pwd,'data/pig');

%1     'Trap__M0_N457_T46_maxG80_slew50_b526_b_eig1_1_1_Ax0_Ay0_Az0_g_M0.mat', ...
%2     'Trap__M0_N571_T57_maxG54_slew34_b526_b_eig1_1_1_Ax0_Ay0_Az0_g_M0.mat', ...
%3     'Trap__M1_N571_T57_maxG80_slew50_b526_b_eig1_1_1_Ax0_Ay0_Az0_g_M1.mat', ...
%4     'Trap__M0_N671_T67_maxG42_slew26_b526_b_eig1_1_1_Ax0_Ay0_Az0_g_M0.mat', ...
%5     'Trap__M1_N671_T67_maxG58_slew36_b525_b_eig1_1_1_Ax0_Ay0_Az0_g_M1.mat', ...
%6     'Trap__M2_N671_T67_maxG80_slew50_b525_b_eig1_1_1_Ax0_Ay0_Az0_g_M2.mat', ...
%7     'Trap_PGSE_LTE_b663_M0_N457_T46_maxG38_slew12_b526_b_eig1_0_0_Ax0_Ay0_Az0_g_M0.mat', ...
%8     'Trap_PGSE_LTE_b663_M0_N571_T57_maxG27_slew7_b525_b_eig1_0_0_Ax0_Ay0_Az0_g_M0.mat', ...
%9     'Trap_BP_LTE_b732_M1_N571_T57_maxG63_slew37_b525_b_eig1_0_0_Ax0_Ay0_Az0_g_M1.mat',...
%10    'Trap_Welsh_LTE_b1086_M2_N571_T57_maxG73_slew55_b526_b_eig1_0_0_Ax0_Ay0_Az0_g_M2.mat', ...
%11    'Trap_PGSE_LTE_b663_M0_N671_T67_maxG21_slew4_b525_b_eig1_0_0_Ax0_Ay0_Az0_g_M0.mat', ...
%12     'Trap_BP_LTE_b732_M1_N671_T67_maxG47_slew24_b525_b_eig1_0_0_Ax0_Ay0_Az0_g_M1.mat', ...
%13     'Trap_Welsh_LTE_b1086_M2_N671_T67_maxG57_slew35_b525_b_eig1_0_0_Ax0_Ay0_Az0_g_M2.mat'};

showLabels = 0;
showLegend = 0;

saveFit = 0; % if 0 then load saved

% ------------------- change ROI here ------------------
dataName = 'heart1';
dataName = 'heart2';

% fit all or only a selected range
fitName = '_FitAll';
fitName = '_FitLong'; % only long encoding time

sl = 3; % slice


load(fullfile(data_subdir,[dataName '.mat']))
if strcmp(dataName,'heart1')
    data = data.pig_01;
elseif strcmp(dataName,'heart2')
    data = data.pig_02;
end

% load table of MD(R) for cylinders
load(fullfile(data_dir, ['lookup_table_shape2_Ndir1000_pig.mat']), 'P', 'R')

% color order
order = [6 5 3 4 2 1 13 12 10 8 11 9 7];

MDexp = squeeze(data.adc_mean(sl,4,:))*1e-6;
MDexp = MDexp(order);

maxN = max(P.Ns);
minN = min(P.Ns);
for n = 1: length(P.isLTE)
    if P.isLTE(n) == 1
        lines{n} = '--';
        markers{n} = 'x';
        MarkerSize(n) = 12;
    else
        lines{n} = '-';
        markers{n} = 'o';
        MarkerSize(n) = 9;
    end
    
    factor = (P.Ns(n)-minN)/(maxN-minN) ;%(P.Ns(n)/maxN)^2;
    
    switch P.M(n)
        case 0
            colors(n,:) = [1 0 0]*factor;
        case 1
            colors(n,:) = [0 1 0]*factor;
        case 2
            colors(n,:) = [0 0 1]*factor;
    end
    
end

lw = 2;
fs = 16;
Xmax = 30;

%N671: STE (M0,M1,M2) and LTE (M0,M1,M2)
%N571: STE (M0,M1) and LTE (M0,M1,M2)
%N457: STE (M0) and LTE (M0,M1,M2)


%--------------- select for fitting

% fitName = '_FitLong';
% % select data
indN = P.Ns == 671;% N = 457 571 671
ind = find(indN); % only N
% % ind = find(P.isLTE); % LTE
% % ind = find(~P.isLTE); % STE
% %ind = find(P.isLTE & indN); % LTE & N
% %ind = find(~P.isLTE & indN); % STE & N
% %ind = find(P.M == 2 & indN); % gradient moment
%
% %ind = find(indN & P.isLTE); % N, LTE
% %ind = find(indN & P.isLTE & P.M == 2); % N, LTE, M
%
% fitName = '_FitAll';
% ind = 1:size(R.MD,2); % all


MD_selected = R.MD(:,ind);
MDexp_selected = MDexp(ind);
Times_selected = P.dt*P.Ns(ind);
lines_selected = lines(ind);
colors_selected = colors(ind,:);
markers_selected = markers(ind);
MarkerSize_selected = MarkerSize(ind);

%--------------- fit ------------
Nfit = 50*10;
fh = linspace(0,1,Nfit);
Dh = linspace(1e-10,2e-9,Nfit);
D0 = 2e-9;

fit_data_path = fullfile(data_subdir,[dataName fitName])
if saveFit
    tic
    [fitData.indR fitData.fh fitData.Dh fitData.RMSE] = fFit(MD_selected, MDexp_selected, fh, Dh, D0/P.D0);
    toc
    save(fit_data_path,'fitData')
else
    load(fit_data_path,'fitData')
end
indR = fitData.indR;
fh = fitData.fh;
Dh = fitData.Dh;
RMSE = fitData.RMSE;

Y = fh*Dh + (1-fh)*MD_selected/P.D0*D0;  % calculated
%residuals = MDexp_selected - Y(indR,:)';

% heart 1
% fit all
% R = 6.11e-06, fres = 0.315, Dh = 4.2e-10, D0 = 2e-09
% RMSE = 1.04e-11, rRMSE = 1.52, sum residuals = 2.47e-12
% fit LTE
% R = 9.94e-06, fres = 0.313, Dh = 2.64e-10, D0 = 2e-09
% RMSE = 2.31e-11, rRMSE = 3.39, sum residuals = 1.74e-10
% fit STE
% R = 6.89e-06, fres = 0.329, Dh = 3.67e-10, D0 = 2e-09
% RMSE = 1.25e-11, rRMSE = 1.84, sum residuals = -5.54e-11

% only long TE
% R = 6.2e-06, fres = 0.323, Dh = 3.97e-10, D0 = 2e-09
% RMSE = 1.18e-11, rRMSE = 1.73, sum residuals = 6.78e-11

% heart 2
% fit all
% R = 7.48e-06, fres = 0.325, Dh = 3.21e-10, D0 = 2e-09
% RMSE = 1.36e-11, rRMSE = 2.04, sum residuals = 1.41e-12
% fit LTE
% R = 1.1e-05, fres = 0.369, Dh = 1e-10, D0 = 2e-09
% RMSE = 2.21e-11, rRMSE = 3.31, sum residuals = 1.83e-10
% fit STE
% R = 5.52e-06, fres = 0.253, Dh = 5.15e-10, D0 = 2e-09
% RMSE = 2.64e-11, rRMSE = 3.95, sum residuals = -1.64e-10

% only long TE
% R = 7.78e-06, fres = 0.339, Dh = 2.83e-10, D0 = 2e-09
% RMSE = 1.39e-11, rRMSE = 2.08, sum residuals = 2.28e-11


Rfit = P.Rs(indR);
%---------------------------

X = 1e6*P.Rs; % mum scale
Y = Y*1e9;


%XTicks = [0.1 0.5 1 2 3 4 5 6 8 10 20 30 40 50 100];
XTicks = [1 2 5 10 20 30 40 50];
Xlabel = 'R [\mum]';
Ylabel = 'MD';

if std(Times_selected) == 0
    title = ['TE = ' num2str(mean(Times_selected)*1e3,3) ', R = ' num2str(Rfit,3) ', fres = ' num2str(1-fh,3) ', Dh = ' num2str(Dh,3) ', D0 = ' num2str(D0,2)];
else
    title = ['R = ' num2str(Rfit,3) ', fres = ' num2str(1-fh,3) ', Dh = ' num2str(Dh,3) ', D0 = ' num2str(D0,2)];
end

fPLot(1,1,0,X,Y,XTicks,Xlabel,Ylabel,min(X),max(X),1*min(Y(:)),1*max(Y(:)),lines_selected, colors_selected, title, showLabels,lw,fs)
xlim([0 Xmax])

if showLegend
    lh = legend;
    legend boxoff
    lh.ItemTokenSize(1) = lh.ItemTokenSize(1)*1.2;
end

hold on
for n = 1:length(MDexp_selected)
    plot(X(indR),MDexp_selected(n)*1e9,markers_selected{n},'Color',colors_selected(n,:),'MarkerSize',MarkerSize_selected(n),'LineWidth',lw)
end
ylim([0.99*min(Y(:)) 1.01*max(Y(:))])
xlim([0 Xmax])
pbaspect([1 1 1])



%--------------- select
%
% select data
% indN = P.Ns == 671;% N = 457 571 671
% ind = find(indN); % only N
%ind = find(P.isLTE); % LTE
%ind = find(~P.isLTE); % STE
%ind = find(P.isLTE & indN); % LTE & N
%ind = find(~P.isLTE & indN); % STE & N
%ind = find(P.M == 2 & indN); % gradient moment

%ind = find(indN & P.isLTE); % N, LTE
%ind = find(indN & P.isLTE & P.M == 2); % N, LTE, M

ind = 1:size(R.MD,2); %all

MD_selected = R.MD(:,ind);
MDexp_selected = MDexp(ind);
Times_selected = P.dt*P.Ns(ind);
lines_selected = lines(ind);
colors_selected = colors(ind,:);
markers_selected = markers(ind);
MarkerSize_selected = MarkerSize(ind);

Ycalc = fh*Dh + (1-fh)*MD_selected/P.D0*D0;  % calculated
residuals = MDexp_selected - Ycalc(indR,:)';


X = 1e6*P.Rs; % mum scale
Y = Ycalc*1e9;

%XTicks = [0.1 0.5 1 2 3 4 5 6 8 10 20 30 40 50 100];
XTicks = [1 2 5 10 20 30 40 50];
Xlabel = 'R [\mum]';
Ylabel = 'MD';

if std(Times_selected) == 0
    title = ['TE = ' num2str(mean(Times_selected)*1e3,3) ', R = ' num2str(Rfit,3) ', fres = ' num2str(1-fh,3) ', Dh = ' num2str(Dh,3) ', D0 = ' num2str(D0,2)];
else
    title = ['R = ' num2str(Rfit,3) ', fres = ' num2str(1-fh,3) ', Dh = ' num2str(Dh,3) ', D0 = ' num2str(D0,2)];
end


fPLot(2,1,0,X,Y,XTicks,Xlabel,Ylabel,min(X),max(X),1*min(Y(:)),1*max(Y(:)),lines_selected, colors_selected, title, showLabels,lw,fs)
hold on
for n = 1:length(MDexp_selected)
    plot(X(indR),MDexp_selected(n)*1e9,markers_selected{n},'Color',colors_selected(n,:),'MarkerSize',MarkerSize_selected(n),'LineWidth',lw)
end
xlim([0 Xmax])
ylim([0.99*min(Y(:)) 1.01*max(Y(:))])
pbaspect([1 1 1])


XTicks = [];
Xlabel = 'MDexp*1e-9';
Ylabel = '(MDexp-MDcalc)*1e-9';

if showLegend
    legend
    legend boxoff
end


X = MDexp_selected*1e9;
Y = residuals'*1e9;

figure(3),clf
for n = 1:length(residuals)
    hold on
    plot(X(n),Y(n),markers_selected{n},'Color',colors_selected(n,:),'MarkerSize',MarkerSize_selected(n),'LineWidth',lw)
end
set(gca,'LineWidth',3,'Box','off','TickDir','out','TickLength',[.02 .02],'FontSize',fs)

if showLabels
    xlabel(Xlabel), ylabel(Ylabel)
end
xlim([0.95*min(X) 1.05*max(X)])

ylim([0.95*min(Y) 1.05*max(Y)])
pbaspect([1.62 1 1])

if showLegend
    legend
    legend boxoff
end

% disp(title)
% RMSE = sqrt(sum(residuals.^2)/length(residuals));
% disp(['RMSE = ' num2str(RMSE,3) ', rRMSE = ' num2str(RMSE/mean(MDexp)*100,3) ', sum residuals = ' num2str(sum(residuals),3) ])

% correlation - errors

fh = figure(4);,clf
hold on
Xplot = MDexp_selected*1e9;
% residuals = MDexp_selected - Y(indR,:)';
Yplot = Ycalc(indR,:)'*1e9;
Error = abs(residuals)'*1e9;

for n = 1:length(residuals)
    
    ph(n) = plot(Xplot(n),Yplot(n),markers_selected{n},'Color',colors_selected(n,:),'MarkerSize',MarkerSize_selected(n),'LineWidth',2);
    errorbar(Xplot(n),Yplot(n),Error(n),markers_selected{n},'Color',colors_selected(n,:),'MarkerSize',MarkerSize_selected(n),'LineWidth',2)
end
set(gca,'LineWidth',3,'Box','off','TickDir','out','TickLength',[.02 .02],'FontSize',fs)

xlim([0.95*min([Xplot; Yplot]) 1.05*max([Xplot; Yplot])])
ylim([0.95*min([Xplot; Yplot]) 1.05*max([Xplot; Yplot])])

Xplot = [min(Yplot) max(Yplot)];
plot(Xplot,Xplot,':k','LineWidth',1);

pbaspect([1 1 1])


if showLabels
    xlabel(Xlabel), ylabel(Ylabel)
end

if showLegend
    legend
    legend boxoff
end

display(title)

RMSE = sqrt(sum(residuals.^2)/length(residuals));
display(['RMSE = ' num2str(RMSE,3) ', rRMSE = ' num2str(RMSE/mean(MDexp)*100,3) ', sum residuals = ' num2str(sum(residuals),3) ])



% ------------- functions ---------------------------------------------

function fPLot(num,semilogX, semilogY, X,Y, XTicks,Xlabel,Ylabel,Xmin,Xmax,Ymin,Ymax, lines, colors, titleString, showLabels,lw,fs)
figure(num),clf
hold on
for n = 1:size(Y,2)
    plot(X,Y(:,n),'LineStyle',lines{n},'Color',colors(n,:),'LineWidth',lw,'MarkerSize',16)
end


if semilogX & ~semilogY
    set(gca, 'XScale', 'log')
end

if ~semilogX & semilogY
    set(gca, 'YScale', 'log')
end

if semilogX & semilogY
    set(gca, 'XScale', 'log')
    set(gca, 'YScale', 'log')
end


set(gca,'LineWidth',3,'Box','off','TickDir','out','TickLength',[.02 .02],'FontSize',fs)

if ~isempty(XTicks)
    set(gca,'XTick',XTicks)
end
if showLabels
    xlabel(Xlabel), ylabel(Ylabel)
end
xlim([Xmin Xmax])
ylim([Ymin Ymax])
%axis tight
if ~isempty(titleString) && showLabels
    th = title(titleString);
    th.FontSize = th.FontSize/2;
end
end

function [nR, fh, Dh, RMSE] = fFit(MD, MDexp, fh, Dh, Dratio)
NR = size(MD,1);
MDexp_mat = repmat(MDexp,1,NR)';

for nf = 1:length(fh)
    for nD = 1: length(Dh)
        Dm = fh(nf)*Dh(nD) + (1-fh(nf))*MD*Dratio;
        MSDif(:,nf,nD) = sum((MDexp_mat- Dm).^2,2);
    end
end

minimum=min(MSDif(:));
%convert linear indices to subscript indices
[nR,minF,minD]=ind2sub(size(MSDif),find(MSDif==minimum));

if length(nR)>1
    disp('more than 1 minima');
    nR = nR(1);
    minF = minF(1);
    minD = minD(1);
end

fh = fh(minF);
Dh = Dh(minD);

RMSE = sqrt(MSDif(nR,minF,minD)/length(MDexp));

end

