% Calculate lookup table: MD vs cylinder R vs waveform
% The table is used for fitting and is saved in the *data* folder.

% this is an older script, which does not use spectral trace for MD calculation
% it relies on powder averaging across 1000 directions
% a much more efficient and generally applicable approach is to use spectral trace (see make_lookup_table_DvsR_mouse.m and noise simulations)

clear all
%close all
restoredefaultpath
addpath(genpath(fullfile(pwd,'add')));

data_dir = fullfile(pwd,'data');

wfm_dir = fullfile(pwd,'wfm_clinical');
P.waveform_names = {...
    'Trap__M0_N457_T46_maxG80_slew50_b526_b_eig1_1_1_Ax0_Ay0_Az0_g_M0.mat', ...
    'Trap__M0_N571_T57_maxG54_slew34_b526_b_eig1_1_1_Ax0_Ay0_Az0_g_M0.mat', ...
    'Trap__M1_N571_T57_maxG80_slew50_b526_b_eig1_1_1_Ax0_Ay0_Az0_g_M1.mat', ...
    'Trap__M0_N671_T67_maxG42_slew26_b526_b_eig1_1_1_Ax0_Ay0_Az0_g_M0.mat', ...
    'Trap__M1_N671_T67_maxG58_slew36_b525_b_eig1_1_1_Ax0_Ay0_Az0_g_M1.mat', ...
    'Trap__M2_N671_T67_maxG80_slew50_b525_b_eig1_1_1_Ax0_Ay0_Az0_g_M2.mat', ...
    'Trap_PGSE_LTE_b663_M0_N457_T46_maxG38_slew12_b526_b_eig1_0_0_Ax0_Ay0_Az0_g_M0.mat', ...
    'Trap_PGSE_LTE_b663_M0_N571_T57_maxG27_slew7_b525_b_eig1_0_0_Ax0_Ay0_Az0_g_M0.mat', ...
    'Trap_BP_LTE_b732_M1_N571_T57_maxG63_slew37_b525_b_eig1_0_0_Ax0_Ay0_Az0_g_M1.mat',...
    'Trap_Welsh_LTE_b1086_M2_N571_T57_maxG73_slew55_b526_b_eig1_0_0_Ax0_Ay0_Az0_g_M2.mat', ...
    'Trap_PGSE_LTE_b663_M0_N671_T67_maxG21_slew4_b525_b_eig1_0_0_Ax0_Ay0_Az0_g_M0.mat', ...
    'Trap_BP_LTE_b732_M1_N671_T67_maxG47_slew24_b525_b_eig1_0_0_Ax0_Ay0_Az0_g_M1.mat', ...
    'Trap_Welsh_LTE_b1086_M2_N671_T67_maxG57_slew35_b525_b_eig1_0_0_Ax0_Ay0_Az0_g_M2.mat'};

P.isLTE = [0 0 0 0 0 0 1 1 1 1 1 1 1];
P.M =     [0 0 1 0 1 2 0 0 1 2 0 1 2];

%P.waveform_names = P.waveform_names(10);

%N671: STE (M0,M1,M2) and LTE (M0,M1,M2)
%N571: STE (M0,M1) and LTE (M0,M1,M2)
%N457: STE (M0) and LTE (M0,M1,M2)


P.dt = 1e-4; % the original sampling interval

Calculate = 1;
calc_name = 'pig';

P.shape = 2; % axial symmetry needed for analytical expressions:  1- sphere, 2- cylinder, 3-ellipsoid
P.D0 = 1e-9;

P.R = 2.5e-6; %restriction sphere

P.Rs = 1e-6*linspace(1,50,500);

P.Nwaveforms = length(P.waveform_names);
for n = 1:P.Nwaveforms
    load(fullfile(wfm_dir,P.waveform_names{n}));
    P.Ns(n) = length(g);
end


P.Tmax = 100*P.dt*max(2*P.Ns); %FT grid

P.NT = round(P.Tmax/P.dt);
P.time = (0:P.NT-1)'*P.dt;


gmr = 26.75e7; % gamma (gyromagnetic ratio)
load UDSRTriN1000 %UDSRTriN30 UDSRTriN1000
C = UDSR.phi;
B = UDSR.theta;

CalcName = ['lookup_table_shape' num2str(P.shape) '_Ndir' num2str(UDSR.N) '_' calc_name ];


if Calculate
    tic
    
    f = 1/P.dt*(0:(P.NT/2))/P.NT;
    f = [-fliplr(f(2:end)) f(1:end)];
    
    
    for nR = 1:length(P.Rs)
        ['nR/N = ' num2str(nR) '/' num2str(length(P.Rs))]
        
        
        switch P.shape
            case 1
                % sphere
                L(:,1) = DwSpherical(2*pi*f,P.Rs(nR),P.D0,0,50)';
                L(:,2) = L(:,1);
                L(:,3) = L(:,1);
                
            case 2
                % cylinder
                L(:,1) = DwCylindrical(2*pi*f,P.Rs(nR),P.D0,0,50)';
                L(:,2) = L(:,1);
                L(:,3) = P.D0*ones(size(L(:,1)));
                
                
            case 3
                % spheroid
                L(:,1) = DwSpherical(2*pi*f,P.Rs(nR),P.D0,0,50)';
                L(:,2) = L(:,1);
                L(:,3) = DwSpherical(2*pi*f,5*P.Rs(nR),P.D0,0,50)';
                
        end
        
        
        
        for nWave = 1:P.Nwaveforms%P.NTE
            
            load(fullfile(wfm_dir,P.waveform_names{nWave}));
            
            Nt = length(g); %number of time steps in the waveform without 0-padding
            tE = P.dt*Nt; %duration of the waveform without 0-padding
            
            G = g;
            F = gmr*P.dt*cumsum(G);
            
            % zero padding
            N0 = P.NT-Nt;
            F_zeropad = [F; zeros(N0,3)];
            
            [f, PS] = fPS(F_zeropad,P.dt);
            % normalize power
            b = real(squeeze(sum(PS)));
            PS = PS/trace(b);
            
            
            % ---------------- attenuation -----------------------
            % assuming normalization of PS
            % if LTE
            if det(b)==0
                [~,ind] = max([b(1,1) b(2,2) b(3,3)]);
                PS = squeeze(PS(:,ind,ind));
                
                beta1D_z = sum(PS.*L(:,3)).*cos(B).^2;
                beta1D_x = sum(PS.*L(:,1)).*sin(B).^2;
                beta = real(beta1D_z + beta1D_x);
                
                % theoretical powder for axial symmetry (no approximation with limited number of directions)
                ADCx = sum(PS.*L(:,1));
                ADCz = sum(PS.*L(:,3));
                
                % powder average result
                R.PE_MD(nR, nWave) = (2*ADCx + ADCz)/3;
                R.PE_V(nR, nWave) = 4/5*(ADCz - ADCx)^2/3^2; %mu2
                
            else % assuming STE here
                % numerical powder average
                beta3D_cross_z =  - sum(PS(:,1,2).*L(:,3) + PS(:,2,1).*L(:,3)).*cos(C).*sin(B).^2.*sin(C) - sum(PS(:,1,3).*L(:,3) + PS(:,3,1).*L(:,3)).*cos(B).*cos(C).*sin(B) + sum(PS(:,2,3).*L(:,3) + PS(:,3,2).*L(:,3)).*cos(B).*sin(B).*sin(C);
                beta3D_z =  sum(PS(:,3,3).*L(:,3)).*(1-sin(B).^2)+ sum(PS(:,1,1).*L(:,3)).*cos(C).^2.*sin(B).^2 + sum(PS(:,2,2).*L(:,3)).*sin(B).^2.*sin(C).^2;
                beta3D_rest = sum(PS(:,2,2).*L(:,1)).*(1-sin(B).^2.*sin(C).^2)  + sum(PS(:,3,3).*L(:,1)).*sin(B).^2  + sum(PS(:,1,1).*L(:,1)).*(sin(C).^2 + cos(B).^2.*cos(C).^2) +...
                    sum((PS(:,1,2)+PS(:,2,1)).*L(:,1)).*cos(C).*sin(B).^2.*sin(C) + sum((PS(:,1,2)+PS(:,3,1)).*L(:,1)).*cos(B).*cos(C).*sin(B) - sum((PS(:,2,3)+PS(:,3,2)).*L(:,1)).*cos(B).*sin(B).*sin(C);
                
                %beta3D = 1/(b.x+b.y+b.z)*(beta3D_z + beta3D_cross_z + beta3D_rest);
                beta = real(beta3D_z + beta3D_cross_z + beta3D_rest);
                
                % assuming normalization
                ADCxx = 3/2*sum((PS(:,2,2)+PS(:,1,1)).*L(:,1));
                ADCzx = 3/2*sum((PS(:,2,2)+PS(:,1,1)).*L(:,3));
                ADCxz = 3*sum(PS(:,3,3).*L(:,1));
                ADCzz = 3*sum(PS(:,3,3).*L(:,3));
                
                R.PE_MD(nR, nWave) = (2*(2*ADCxx + ADCxz)+2*ADCzx + ADCzz)/9;
                R.PE_V(nR, nWave) = 4/5*(ADCzz - ADCzx + ADCxx - ADCxz)^2/9^2; %mu2
                
            end
            
            R.MD(nR, nWave) = mean(beta);
            R.MD2(nR, nWave) = mean(beta.^2);
            R.V(nR, nWave) = mean((beta - R.MD(nR,nWave)).^2);
            
        end
    end
    save(fullfile(data_dir, [CalcName '.mat']), 'P', 'R')
    toc
    
else
    load(fullfile(data_dir, [CalcName '.mat']), 'P', 'R')
end



% --------------- plotting -----------------------

Y1 = [R.MD]/P.D0; % only numerical powders
Y2 = [R.V]/P.D0^2;

Y1 = [R.MD(:,find(P.isLTE))]/P.D0; % LTE
Y2 = [R.V(:,find(P.isLTE))]/P.D0^2;

Y1 = [R.MD(:,find(~P.isLTE))]/P.D0; % STE
Y2 = [R.V(:,find(~P.isLTE))]/P.D0^2;

ind = P.Ns == 671;% 457 571 671
Y1 = [R.MD(:,find(P.isLTE & ind))]/P.D0; % LTE & N
Y2 = [R.V(:,find(P.isLTE & ind))]/P.D0^2;

% Y1 = [R.MD(:,find(~P.isLTE & ind))]/P.D0; % STE & N
% Y2 = [R.V(:,find(~P.isLTE & ind))]/P.D0^2;

ind = P.Ns == 671;% 457 571 671
Y1 = [R.MD(:,find(ind))]/P.D0; % N
Y2 = [R.V(:,find(ind))]/P.D0^2;

ind1 = P.Ns == 671;% 457 571 671
ind2 = P.M == 1; %0 1 2
Y1 = [R.MD(:,find(ind1 & ind2))]/P.D0; % N
Y2 = [R.V(:,find(ind1 & ind2))]/P.D0^2;

fig_offeset = 0*2;
X = P.Rs*1e6;
%XTicks = [1 10:10:60];
XTicks = [1 2 5 10:10:60];
Xlabel = 'R [mum]';
Ylabel = 'MD / D_0';
fPLot(1+fig_offeset,X,Y1,XTicks,Xlabel,Ylabel,min(X),1.05*max(X),0,1,[])

% Ylabel = 'V / D_0^2';
% fPLot(2+fig_offeset,X,Y2,XTicks,Xlabel,Ylabel,min(X),max(X),0,max(Y2(:)),[])

% Y1 = [R.MD; R.PE_MD]/P.D0; % compare theoretical and numerical powders
% Y2 = [R.V; R.PE_V]/P.D0^2;

% ------------- functions ---------------------------------------------


function [f PS] = fPS(F,dt,mode)
%  Power Spectra of F

N = size(F,2);
NT = size(F,1);

f = 1/dt*(0:(NT/2))/NT;

if (exist('mode', 'var'))
    if strcmp(mode,'positive')
        mode = 1;
        PS = zeros(length(f),N,N);
    end
else
    mode = 0;
end

if ~mode
    f = [-fliplr(f(2:end)) f(1:end)];
    PS = zeros(NT+1,N,N);
end


for m = 1:N
    for n = 1:N
        
        ps = fft(F(:,m)).*conj(fft(F(:,n)));
        
        if mode
            PStmp = ps(1:length(f));
        else
            PStmp = fftshift(ps);
            PStmp = [PStmp; PStmp(1)];
        end
        %         figure(1),clf
        %         plot(f,real(PStmp),f,imag(PStmp))
        %         sum(real(PStmp))
        %         sum(imag(PStmp))
        PS(:,m,n) = PStmp;
    end
end
end

function fPLot(num,X,Y,XTicks,Xlabel,Ylabel,Xmin,Xmax,Ymin,Ymax,titleString)
figure(num),clf
semilogx(X,Y,'-','LineWidth',3,'MarkerSize',10);
%plot(X,Y,'-','LineWidth',3,'MarkerSize',10);
set(gca,'LineWidth',3,'Box','off','TickDir','out','TickLength',[.02 .02],'FontSize',14)

if ~isempty(XTicks)
    set(gca,'XTick',XTicks)
end
xlabel(Xlabel), ylabel(Ylabel)
xlim([Xmin Xmax])
ylim([Ymin Ymax])
%axis tight
if ~isempty(titleString)
    title(titleString)
end
end




