% Perform noise simulations, calculate signal error as a function of *f<sub>R</sub>* and SNR and generate figures for cylinders.
% Makes *_noise_fig_data.mat files in the *model* folder.

clear all
close all

noise_dir = 'noise_simulations';

saveFigData = 0; % if 1 run simulation, else read and plot results
FigDataName = '';

saveFig = 0;
fig_resolution = '-r300'; % dpi, e.g. '-r72' '-r600'


model_dir = fullfile(pwd,'model');
substrateName = 'DvsR_cylinder';

load(fullfile(model_dir,[substrateName '.mat']),'R')


% waveforms
% 1. M2_LTE_1, 2. M2_LTE_2, 3. M2_LTE_3, 4. M2_STE
% 1. M2_LTE_1=M2_LTE_2, M2_LTE_3 - detuned,

% ---------- select waveforms -------
%{'PGSE_LTE'  'M0_LTE_1'  'M0_LTE_2'  'M0_LTE_3'  'M0_STE'  'M1_LTE_1'  'M1_LTE_2'  'M1_LTE_3'  'M1_STE'  'M2_LTE_1'  'M2_LTE_2'  'M2_LTE_3'  'M2_STE'}

%'LTE_3' - 'STE'
db_ind(1,:) = 4+[0 4 4*2]; %LTE3
db_ind(2,:) = 5+[0 4 4*2]; % STE

%'LTE_3' - 'LTE_1'
LTE_ind(1,:) = 4+[0 4 4*2]; %LTE3
LTE_ind(2,:) = 2+[0 4 4*2]; %LTE1

% qMAS
% {'qMAS_detunedLTE_SE_flip1', 'qMAS_tunedLTE_SE_flip1', 'qMAS_SE_flip1'};

%'qMAS_tunedLTE_SE_flip1' - 'qMAS_SE_flip1'
% db_ind = ...
%     [2;...
%     3];

%'qMAS_detunedLTE_SE_flip1' - 'qMAS_tunedLTE_SE_flip1
% LTE_ind = ...
%     [1;...
%     2];


if saveFigData
    
    P.NdS = size(db_ind,2);
    
    
    P.Ndir = size(R.ADCs,3);
    P.SNR = [linspace(1,301,301) Inf];
    
    P.NSNR = 1000; %10000;
    
    P.fres = [.25 .5 .75 1];
    P.Dh = 1e-9; % hindered Gaussian
    P.b = 2e9;
    
    P.Nb = length(P.b);
    P.Nf = length(P.fres);
    P.NSNRs = length(P.SNR);
    P.Nr = length(R.Rs);
    P.Nwf = size(R.ADCs,1);
    P.Ndir = size(R.ADCs,3);
    
    
    ADCs = reshape(R.ADCs,P.Nwf,P.Nr,P.Ndir);
    P.MD = reshape(R.MD,P.Nwf,P.Nr);
    P.VA = reshape(R.VA,P.Nwf,P.Nr);
    
    %------- signal --------
    Dhs = ones(size(ADCs))*P.Dh;
    
    count = 0;
    for nb = 1:P.Nb
        for nf = 1:P.Nf
            for nSNR = 1:P.NSNRs
                count = count +1;
                display(sprintf('%.2f %%', count/P.Nb/P.Nf/P.NSNRs*100))
                
                Einf = (1-P.fres(nf))*exp(-P.b(nb)*Dhs) + P.fres(nf)*exp(-P.b(nb)*ADCs);
                
                
                if P.SNR(nSNR) == Inf
                    
                    P.S(nb,nf,nSNR,:,:) = mean(Einf,3);
                    P.eS(nb,nf,nSNR,:,:) = R.MD*0;
                    
                    
                    for ndS = 1:P.NdS
                        dE_dbinf = squeeze( Einf(db_ind(1,ndS),:,:) - Einf(db_ind(2,ndS),:,:) ); %  LTEtuned - STE
                        dE_LTEinf = squeeze( Einf(LTE_ind(1,ndS),:,:) - Einf(LTE_ind(2,ndS),:,:) ); % LTEdetuned - LTEtuned
                        
                        P.dS_db(nb,nf,nSNR,ndS,:) = mean(dE_dbinf,2);
                        P.dS_LTE(nb,nf,nSNR,ndS,:) = mean(dE_LTEinf,2);
                    end
                    
                    
                else
                    cum_mS = 0;
                    cum_mS2 = 0;
                    
                    for n = 1:P.NSNR
                        Snoise = sqrt((Einf + 1/P.SNR(nSNR)*randn(size(Einf))).^2+(1/P.SNR(nSNR)*randn(size(Einf))).^2);
                        
                        mS1 = mean(Snoise,3);
                        cum_mS = cum_mS + mS1;
                    end
                    P.S(nb,nf,nSNR,:,:) = cum_mS/P.NSNR;
                    P.eS(nb,nf,nSNR,:,:) = sqrt(abs( cum_mS2/P.NSNR - (cum_mS/P.NSNR).^2 ));
                    
                    for ndS = 1:P.NdS
                        
                        cum_m_dS_db = 0;
                        cum_m_dS_db2 = 0;
                        cum_m_dS_LTE = 0;
                        cum_m_dS_LTE2 = 0;
                        
                        
                        for n = 1:P.NSNR
                            Snoise = sqrt((Einf + 1/P.SNR(nSNR)*randn(size(Einf))).^2+(1/P.SNR(nSNR)*randn(size(Einf))).^2);
                            
                            mS1 = mean(Snoise,3);
                            cum_mS = cum_mS + mS1;
                            cum_mS2 = cum_mS2 + mS1.^2;
                            
                            
                            dS_db1 = squeeze( Snoise(db_ind(1,ndS),:,:) - Snoise(db_ind(2,ndS),:,:) ); % LTEtuned - STE
                            m_dS_db1 = mean(dS_db1,2);
                            dS_LTE1 = squeeze( Snoise(LTE_ind(1,ndS),:,:) - Snoise(LTE_ind(2,ndS),:,:) ); % LTEdetuned - LTEtuned
                            m_dS_LTE1 = mean(dS_LTE1,2);
                            
                            cum_m_dS_db = cum_m_dS_db + m_dS_db1;
                            cum_m_dS_db2 = cum_m_dS_db2 + m_dS_db1.^2;
                            
                            cum_m_dS_LTE = cum_m_dS_LTE + m_dS_LTE1;
                            cum_m_dS_LTE2 = cum_m_dS_LTE2 + m_dS_LTE1.^2;
                            
                        end
                        
                        
                        P.dS_db(nb,nf,nSNR,ndS,:) = cum_m_dS_db/P.NSNR;
                        P.e_dS_db(nb,nf,nSNR,ndS,:) = sqrt(abs( cum_m_dS_db2/P.NSNR - (cum_m_dS_db/P.NSNR).^2 ));
                        
                        P.dS_LTE(nb,nf,nSNR,ndS,:) = cum_m_dS_LTE/P.NSNR;
                        P.e_dS_LTE(nb,nf,nSNR,ndS,:) = sqrt(abs( cum_m_dS_LTE2/P.NSNR - (cum_m_dS_LTE/P.NSNR).^2 ));
                        
                    end
                end
            end
        end
    end
    save(fullfile(model_dir,[substrateName FigDataName '_noise_fig_data.mat']),'P')
    
else
    load(fullfile(model_dir,[substrateName FigDataName '_noise_fig_data.mat']),'P')
end


% ----------- select waveform
ndS = 2;
col = [1 0 0];
col = [0 .8 0];
% col = [0 0 1];
lineStyle = '-';
% col = .2*[1 1 1];
% lineStyle = ':';
% -------------------------------------

P.dS_db_Inf = squeeze(P.dS_db(:,:,end,ndS,:));
P.dS_LTE_Inf = squeeze(P.dS_LTE(:,:,end,ndS,:));

P.dS_db_sel = squeeze(P.dS_db(:,:,1:end-1,ndS,:));
P.dS_LTE_sel = squeeze(P.dS_LTE(:,:,1:end-1,ndS,:));
P.e_dS_db_sel = squeeze(P.e_dS_db(:,:,:,ndS,:));
P.e_dS_LTE_sel = squeeze(P.e_dS_LTE(:,:,:,ndS,:));


fs = 14;
lw = 3;

showLabels = 0;

fig.H = 650;
fig.W = 400;
fig.fs = 16;
fig.lw = 3;

% ---------- show signal at SNR = Inf ---------------------------------------------
if (0)
    fh = figure(1);
    clf
    fh.Color = [1 1 1];
    pause(.1)
    fh.Position = [800 200 fig.W fig.H];
    subFigH = .4;
    subFigW = .75;
    subFigLeft = .2;
    subFigBottom = [.58 .1];
    
    X = R.Rs*1e6;
    
    axes('position',[subFigLeft subFigBottom(1) subFigW subFigH]);
    
    Y = P.dS_db_Inf;
    plot(X,Y,'LineWidth',lw)
    ylabel('LTE - STE')
    set(gca, 'YScale', 'lin',...
        'FontSize',fig.fs,'box','off','TickDir','out','LineWidth',fig.lw,'TickLength',[.02 .02])
    
    axes('position',[subFigLeft subFigBottom(2) subFigW subFigH]);
    Y = P.dS_LTE_Inf;
    [~,ind] = max(Y');
    Xmax = X(round(mean(ind)));
    
    plot(X,Y,'LineWidth',lw)
    ylabel('LTE difference')
    
    xlabel('R [µm]')
    set(gca, 'YScale', 'lin',...
        'FontSize',fig.fs,'box','off','TickDir','out','LineWidth',fig.lw,'TickLength',[.02 .02])
    title(sprintf('max. at %.2f',Xmax))
end


% ---------- show contrast 1D / var fR ---------------------------------------------
if (1)
    
    nSNR = 50;
    nfs = [1:4];
    
    fh = figure(2);
    clf
    fh.Color = [1 1 1];
    pause(.1)
    fh.Position = [800 200 fig.W fig.H];
    subFigH = .38;
    subFigW = .75;
    subFigLeft = .2;
    subFigBottom = [.58 .1];
    
    XLIM1 = [0 10];
    XLIM2 = XLIM1/sqrt(R.D0*mean(R.TE))*1e-6;
    
    Y = squeeze(P.dS_db_sel(nfs, nSNR,  :)./P.e_dS_db_sel(nfs, nSNR, :)); Y = abs(Y);
    
    % top axis
    X = R.Rs/sqrt(R.D0*mean(R.TE));
    ax2 = axes('position',[subFigLeft subFigBottom(1) subFigW subFigH]);
    ax2.XAxisLocation = 'top';
    set(ax2, 'YScale', 'lin',...
        'FontSize',fig.fs,'box','off','TickDir','out','LineWidth',fig.lw,'TickLength',[.02 .02],'Ytick',[])
    xlim(XLIM2)
    
    
    % bottom axis
    X = R.Rs*1e6;
    ax1 = axes('position',[subFigLeft subFigBottom(1) subFigW subFigH*.99]);
    plot(X,Y,'LineWidth',lw,'color',col,'LineStyle',lineStyle)
    ylabel('LTE - STE')
    set(ax1, 'YScale', 'lin',...
        'FontSize',fig.fs,'box','off','TickDir','out','LineWidth',fig.lw,'TickLength',[.02 .02])
    title(sprintf('SNR = %.0f',P.SNR(nSNR)))
    xlim(XLIM1)
    
    Y = squeeze(P.dS_LTE_sel(nfs, nSNR, :)./P.e_dS_LTE_sel(nfs, nSNR, :)); Y = abs(Y);
    % top axis
    X = R.Rs/sqrt(R.D0*mean(R.TE));
    ax3 = axes('position',[subFigLeft subFigBottom(2) subFigW subFigH]);
    %     plot(X,Y,'LineWidth',lw,'color',col,'LineStyle',lineStyle)
    ax3.XAxisLocation = 'top';
    set(gca, 'YScale', 'lin',...
        'FontSize',fig.fs,'box','off','TickDir','out','LineWidth',fig.lw,'TickLength',[.02 .02],'Ytick',[])
    xlim(XLIM2)
    
    % bottom axis
    X = R.Rs*1e6;
    ax4 = axes('position',[subFigLeft subFigBottom(2) subFigW subFigH*.99]);
    plot(X,Y,'LineWidth',lw,'color',col,'LineStyle',lineStyle)
    ylabel('LTE - STE')
    set(gca, 'YScale', 'lin',...
        'FontSize',fig.fs,'box','off','TickDir','out','LineWidth',fig.lw,'TickLength',[.02 .02])
    %     title(sprintf('SNR = %.0f',P.SNR(nSNR)))
    xlim(XLIM1)
    xlabel('R [µm]')
    ylabel('LTE difference')
    
    return
end


% ------------------------- show contrast 2D --------------------------------
if (1)
    fig_par.fs = 16;
    fig_par.TitleFontSizeMultiplier = 0.95;
    fig_par.NAxesTicks = 7;
    fig_par.NColorbarTicks = 7;
    fig_par.CountourLevel = 3;
    fig_par.CountourLW = 2;
    
    
    fh = figure(2);
    clf
    fh.Color = [1 1 1];
    fig.H = 400;
    fig.W = 666;
    pause(.1)
    fh.Position = [600 5 fig.W fig.H];
    
    
    X = P.SNR(1:end-1);
    Y = R.Rs*1e6;
    Y = [R.Rs*1e6; R.Rs/sqrt(R.D0*mean(R.TE))];
    
    fig_par.Grid = [3 2]; % columns rows
    
    fig_par.figLeft = 0.08; %0.11;
    fig_par.figBottom = 0.05;
    fig_par.subLeft = -0.03; %-0.03;
    fig_par.subRight = 0.015;
    fig_par.subBottom = 0.09;
    fig_par.subTop = 0.05;
    fig_par.colorbarMove = 0.35;
    
    
    nf = 2;
    
    % ----------- select fraction
    P.dS_db_sel = squeeze(P.dS_db(:,nf,1:end-1,:,:));
    P.dS_LTE_sel = squeeze(P.dS_LTE(:,nf,1:end-1,:,:));
    P.e_dS_db_sel = squeeze(P.e_dS_db(:,nf,:,:,:));
    P.e_dS_LTE_sel = squeeze(P.e_dS_LTE(:,nf,:,:,:));
    
    Zall1 = squeeze(P.dS_db_sel./P.e_dS_db_sel); Zall1 = abs(Zall1);
    Zall2 = squeeze(P.dS_LTE_sel./P.e_dS_LTE_sel); Zall2 = abs(Zall2);
    
    ZLIM1 = [min(Zall1(:)) max(Zall1(:))];
    ZLIM2 = [min(Zall2(:)) max(Zall2(:))];
    ZLIM = [min([ZLIM1(1) ZLIM2(1)]) max([ZLIM1(2) ZLIM2(2)])];
    
    
    for ndS = 1:3
        pos = [ndS 1];
        if ndS == 3
            fig_par.colorbarVisible = 'on';
        else
            fig_par.colorbarVisible = 'off';
        end
        
        
        Y2D = squeeze(Zall1(:,ndS,:))';
        if showLabels
            fig_par.title_str = sprintf('M%d: %s', ndS, 'SdLTE-SSTE / error');
        end
        fh = plot2Daxes(fh, pos, fig_par, Y2D, X, Y, ZLIM1); %[min(Y2D(:)) max(Y2D(:))]);
        
        
        pos = [ndS 2];
        %     fig_par.colorbarVisible = 'off';
        
        Y2D = squeeze(Zall2(:,ndS,:))';
        
        if showLabels
            fig_par.title_str = sprintf('M%d: %s', ndS, 'SdLTE-StSTE / error');
        end
        fh = plot2Daxes(fh, pos, fig_par, Y2D, X, Y, ZLIM2); %[min(Y2D(:)) max(Y2D(:))]);
        
        if ndS == 1 && showLabels == 1
            %             ylabel('R [µm]')
            xlabel('SNR')
        end
        
    end
    
    fileName = sprintf('%s_fR%d_double_raw.png', substrateName, round(P.fres(nf)*100) );
    display(fileName)
    
    
    if saveFig
        fig_path = fullfile(model_dir,fileName);
        
        print(fh,fig_path,'-dpng',fig_resolution);
    end
end





% ------------- functions ---------------------------------------------



function fh = plot2Daxes(fh, position, parameters, Y2D, X, Y, cLIMs)

parameters.size(1) = (1-parameters.figLeft)/parameters.Grid(1);
parameters.size(1) = parameters.size(1) - parameters.subLeft - parameters.subRight;

parameters.size(2) = (1-parameters.figBottom)/parameters.Grid(2);
parameters.size(2) = parameters.size(2) - parameters.subBottom - parameters.subTop;

x = position(1)*parameters.subLeft + (position(1)-1)*parameters.subRight + (position(1)-1)*parameters.size(1);
position(2) = parameters.Grid(2) - position(2) + 1;
y = position(2)*parameters.subBottom + (position(2)-1)*parameters.subTop + (position(2)-1)*parameters.size(2);

sub_position = [...
    parameters.figLeft   + x ...
    parameters.figBottom + y ...
    parameters.size(1) ...
    parameters.size(2)];


if nargin < 6 || isempty(cLIMs)
    clims(1) = min(Y2D(:));
    clims(2) = max(Y2D(:));
else
    clims = cLIMs;
end

if clims(1) > 0 & clims(1)<clims(2)*1e-13
    clims(1) = 0;
end

if clims(1) == clims(2)
    clims(2) = clims(1) + 1e-32;
end

axes('position',sub_position);

if nargin < 4
    imagesc(Y2D, clims)
    hold on
    %contour(Y2D,parameters.CountourLevel*[1 1],'color','w')
    if parameters.CountourLevel ~= 0
        [C,h] = contour(Y2D,parameters.CountourLevel*[1 1],'edgecolor','w', 'LineWidth',parameters.CountourLW);%,'LineWidth',4, 'ShowText','on');
    end
    
    set(gca,'YDir','normal','FontSize',parameters.fs)
    axis  tight %off % equal tight off
    set(gca,'TickDir','out','TickLength',[0.01, 0.01])
else
    hold on
    if size(Y,1) == 1
        imagesc(X,Y,Y2D, clims)
    else
        yyaxis left
        imagesc(X,Y(1,:),Y2D, clims)
        ax = gca;
        ax.YColor = [0 0 0];
        % ylabel('Left Side')
        yyaxis right
        imagesc(X,Y(2,:),Y2D, clims)
        ax.YColor = 0*[1 0.3 0];
        %   ylabel('Right Side')
        Y = Y(2,:);
    end
    
    if parameters.CountourLevel ~= 0
        [C,h] = contour(X,Y,Y2D,parameters.CountourLevel*[1 1],'edgecolor','w', 'LineWidth',parameters.CountourLW);%,'LineWidth',4, 'ShowText','on');
    end
    
    set(gca,'YDir','normal','FontSize',parameters.fs)
    axis tight% equal tight
    set(gca,'TickDir','out','TickLength',[0.01, 0.01])
    
end

colormap(get_cmap(clims))

cb = colorbar();

if isfield(parameters,'colorbarVisible')
    cb.Visible = parameters.colorbarVisible;
end
box off
if isfield(parameters,'title_str')
    title(parameters.title_str)
    fh.Children(2).TitleFontSizeMultiplier = parameters.TitleFontSizeMultiplier;
end
fh.Children(1).Box = 'off';

fh.Children(1).TickDirection = 'out';

fh.Children(1).LineWidth = 2;
fh.Children(2).LineWidth = 2;
end


function cmap = get_cmap(realY2D)
minY = min(realY2D(:));
maxY = max(realY2D(:));
cmap1 = flipud(bone);
cmap2 = copper;

if minY*maxY < 0
    n = round(length(cmap1)*abs(minY)/(abs(minY)+abs(maxY)));
    cmap1 = interp1(1:length(cmap1),cmap1,linspace(1,length(cmap1),n));
    n = round(length(cmap2)*abs(maxY)/(abs(minY)+abs(maxY)));
    cmap2 = interp1(1:length(cmap2),cmap2,linspace(1,length(cmap2),n));
    cmap = [cmap1; cmap2];
else
    cmap = [cmap2];
end
end
