% plot S(b,R) and MD(b,R) for cylinders
% Makes *_signal_fig_data.mat file in the *model* folder.

clear all
close all

saveFigData = 1;
FigDataName = '_signal';

saveFig = 0;
fig_resolution = '-r300'; % dpi, e.g. '-r72' '-r600'

model_dir = fullfile(pwd,'model');
substrateName = 'DvsR_cylinder';
load(fullfile(model_dir,[substrateName '.mat']),'R')

tunedLTE_ind = 2+[0 4 4*2]; %LTE1
detunedLTE_ind = 4+[0 4 4*2]; %LTE3
STE_ind = 5+[0 4 4*2];



if saveFigData
    
    P.b = 2e9*linspace(0,1,300);
    P.binit = P.b; % for plotting inital slpe
    
    P.Nb = length(P.b);
    P.Nr = length(R.Rs);
    P.Nwf = size(R.ADCs,1);
    P.Ndir = size(R.ADCs,3);
    
    ADCs = reshape(R.ADCs,P.Nwf,P.Nr,P.Ndir);
    P.MD = reshape(R.MD,P.Nwf,P.Nr);
    P.VA = reshape(R.VA,P.Nwf,P.Nr);
    
    %------- signal --------
    
    count = 0;
    for nb = 1:P.Nb
        
        count = count +1;
        display(sprintf('%.2f %%', count/P.Nb))
        
        %Einf = (1-P.fres(nf))*exp(-P.b(nb)*Dhs) + P.fres(nf)*exp(-P.b(nb)*ADCs);
        Einf = exp(-P.b(nb)*ADCs);
        P.S(:,:,nb) = mean(Einf,3);
        P.MD(:,:,nb) =-log(P.S(:,:,nb)./P.S(:,:,1))./(P.b(nb)-P.b(1));
        
    end
    
    
    P.MDtrue = -log(P.S(:,:,2)./P.S(:,:,1))/(P.b(2)-P.b(1));
    
    for nb = 1:P.Nb
        P.Sinit(:,:,nb) = exp(-P.MDtrue*P.binit(nb));
    end
    
    save(fullfile(model_dir,[substrateName FigDataName '_fig_data.mat']),'P')
else
    load(fullfile(model_dir,[substrateName FigDataName '_fig_data.mat']),'P')
end





% ---------- show signal vs b ---------------------------------------------
if (1)
    
    
    %lws = [8 6 3];
    lws = [4 4 4];
    
    col = [1 0 0; ...
        0 0 1; ...
        .4 .4 .4];
    
    lineStyle = '-';
    
    
    showLabels = 0;
    
    fig.fs = 16;
    fig.lw = 3;
    
    XLIM = [0 2000];
    
    GR = 2; %1.62;
    
    fig.H = 400;
    fig.W = fig.H*GR;
    
    % select M-nullings
    Ms = [0:2];
    
    for M = Ms
        
        switch M
            case 0
                YLIM = [.11 1];
            case 1
                YLIM = [.08 1];
            case 2
                YLIM = [.05 1];
        end
        
        fh = figure(1);
        clf
        fh.Color = [1 1 1];
        pause(.1)
        fh.Position = [400 100 fig.W fig.H];
        subFigH = .75;
        subFigW = .21;
        subFigLeft = [.1 .08+[1 2]/3];
        subFigBottom = .15;
        
        
        X = P.b*1e-6;
        Xinit = P.binit*1e-6;
        
        
        ind = [tunedLTE_ind(M+1); detunedLTE_ind(M+1); STE_ind(M+1)];
        
        %find max difference signal
        [~,maxR_ind] = max(P.S(ind(1),:,end)-P.S(ind(2),:,end));
        %     R.Rs(maxR_ind)
        
        %Nrs = [11 45 91];
        Nrs = [1 25 64]; %1.0000    2.5251    5.0033
        
        
        R.Rs(Nrs)*1e6
        
        Nr = Nrs(1);
        axes('position',[subFigLeft(1) subFigBottom subFigW subFigH]);
        hold on
        for nwf = 1:3
            Y = squeeze(P.S(ind(nwf),Nr,:));
            plot(X,Y,'LineWidth',lws(nwf),'color',col(nwf,:),'LineStyle',lineStyle)
            Y = squeeze(P.Sinit(ind(nwf),Nr,:));
        end
        for nwf = 1:2
            Y = squeeze(P.Sinit(ind(nwf),Nr,:));
            plot(Xinit,Y,'LineWidth',fig.lw,'color',col(nwf,:),'LineStyle',':');
        end
        
        set(gca, 'YScale', 'log',...
            'FontSize',fig.fs,'box','off','TickDir','out','LineWidth',fig.lw,'TickLength',[.02 .02])
        
        ylim(YLIM)
        xlim(XLIM)
        
        if showLabels
            ylabel('signal')
            xlabel('b [s/mm^2]')
            title(sprintf('R = %.2f um',1e6*R.Rs(Nr)))
        end
        
        Nr = Nrs(2);
        axes('position',[subFigLeft(2) subFigBottom subFigW subFigH]);
        hold on
        for nwf = 1:3
            Y = squeeze(P.S(ind(nwf),Nr,:));
            plot(X,Y,'LineWidth',lws(nwf),'color',col(nwf,:),'LineStyle',lineStyle)
        end
        for nwf = 1:2
            Y = squeeze(P.Sinit(ind(nwf),Nr,:));
            plot(Xinit,Y,'LineWidth',fig.lw,'color',col(nwf,:),'LineStyle',':');
        end
        
        
        set(gca, 'YScale', 'log',...
            'FontSize',fig.fs,'box','off','TickDir','out','LineWidth',fig.lw,'TickLength',[.02 .02])
        
        ylim(YLIM)
        xlim(XLIM)
        
        %     ylabel('signal')
        %     xlabel('b [s/mm^2]')
        if showLabels
            title(sprintf('R = %.2f um',1e6*R.Rs(Nr)))
        end
        
        Nr = Nrs(3);
        axes('position',[subFigLeft(3) subFigBottom subFigW subFigH]);
        hold on
        for nwf = 1:3
            Y = squeeze(P.S(ind(nwf),Nr,:));
            plot(X,Y,'LineWidth',lws(nwf),'color',col(nwf,:),'LineStyle',lineStyle)
        end
        for nwf = 1:2
            Y = squeeze(P.Sinit(ind(nwf),Nr,:));
            plot(Xinit,Y,'LineWidth',fig.lw,'color',col(nwf,:),'LineStyle',':');
        end
        
        set(gca, 'YScale', 'log',...
            'FontSize',fig.fs,'box','off','TickDir','out','LineWidth',fig.lw,'TickLength',[.02 .02])
        %     ylabel('signal')
        %     xlabel('b [s/mm^2]')
        ylim(YLIM)
        xlim(XLIM)
        
        if showLabels
            title(sprintf('R = %.2f um',1e6*R.Rs(Nr)))
        end
        lh = legend();
        lh.Box = 'off';
        lh.String = {'', '', '','',''};
        
        if saveFig
            fig_path = fullfile(model_dir,sprintf('Svsb_%s_M%d_R%d_%d_%d_raw.png', ...
                substrateName, M, round(1e7*R.Rs(Nrs(1))), round(1e7*R.Rs(Nrs(2))), round(1e7*R.Rs(Nrs(3)))));
            
            print(fh,fig_path,'-dpng',fig_resolution);
        end
        
        %return
    end
end

% ---------- show MD vs b vs R ---------------------------------------------

if (1)
    
    fig_par.fs = 16;
    fig_par.TitleFontSizeMultiplier = 0.7; %0.95;
    fig_par.NAxesTicks = 7;
    fig_par.NColorbarTicks = 7;
    fig_par.CountourLevel = 0.85; %1-0.85; %0.85;
    fig_par.CountourLW = 2;
    fig_par.showLabels = showLabels;
    
    fig_par.Grid = [3 2]; % columns rows
    
    fig.H = 400;
    fig.W = 666;
    fig_par.figLeft = 0.11;
    fig_par.figBottom = 0.05;
    fig_par.subLeft = -0.03;
    fig_par.subRight = 0.015;
    fig_par.subBottom = 0.09;
    fig_par.subTop = 0.05;
    
    
    fh = figure(100);
    clf
    fh.Color = [1 1 1];
    pause(.1)
    fh.Position = [400 200 fig.W fig.H];
    
    P.MD = P.MD(:,:,2:end);
    X = P.b(2:end)*1e-6;
    %     Y = R.Rs*1e6;
    %     Y = R.Rs/sqrt(R.D0*mean(R.TE));
    
    Y = [ R.Rs*1e6; R.Rs/sqrt(R.D0*mean(R.TE))];
    
    
    Zall1 = P.MD(detunedLTE_ind,:,:)./P.MD(STE_ind,:,:);
    names1 = {'M0: MDdLTE/MDSTE','M1: MDdLTE/MDSTE','M2: MDdLTE/MDSTE'};
    Zall2 = P.MD(tunedLTE_ind,:,:)./P.MD(STE_ind,:,:);
    names2 = {'M0: MDtLTE/MDSTE','M1: MDtLTE/MDSTE','M2: MDtLTE/MDSTE'};
    
    ZLIM1 = [min(Zall1(:)) max(Zall1(:))];
    ZLIM2 = [min(Zall2(:)) max(Zall2(:))];
    ZLIM = [min([ZLIM1(1) ZLIM2(1)]) max([ZLIM1(2) ZLIM2(2)])];
    
    
    for m = 1:3
        pos = [m 1];
        
        if m == 3
            fig_par.colorbarVisible = 'on';
        else
            fig_par.colorbarVisible = 'off';
        end
        
        Y2D = squeeze(Zall1(m, :,  :));
        
        if showLabels
            fig_par.title_str = sprintf('%s', names1{m});
        end
        fh = plot2Daxes(fh, pos, fig_par, Y2D, X, Y, ZLIM); %[min(Y2D(:)) max(Y2D(:))]);
        
        pos = [m 2];
        
        fig_par.colorbarVisible = 'off';
        
        Y2D = squeeze(Zall2(m, :,  :));
        
        if showLabels
            fig_par.title_str = sprintf('%s', names2{m});
        end
        fh = plot2Daxes(fh, pos, fig_par, Y2D, X, Y, ZLIM); %[min(Y2D(:)) max(Y2D(:))]);
        
        if m == 1 && fig_par.showLabels == 1
            xlabel('b [s/mm^2]')
            if size(Y,1) == 1
                ylabel('R [µm]')
                %ylabel('R / sqrt(D0*tau)')
            end
            %             ylabel('R / sqrt(D0*tau)')
        end
    end
    
    if saveFig
        fig_path = fullfile(model_dir,sprintf('MDdifferences_%s_raw.png', substrateName));
        
        print(fh,fig_path,'-dpng',fig_resolution);
    end
    
end





% ------------- functions ---------------------------------------------

function fh = plot2Daxes(fh, position, parameters, Y2D, X, Y, cLIMs)

parameters.size(1) = (1-parameters.figLeft)/parameters.Grid(1);
parameters.size(1) = parameters.size(1) - parameters.subLeft - parameters.subRight;

parameters.size(2) = (1-parameters.figBottom)/parameters.Grid(2);
parameters.size(2) = parameters.size(2) - parameters.subBottom - parameters.subTop;

x = position(1)*parameters.subLeft + (position(1)-1)*parameters.subRight + (position(1)-1)*parameters.size(1);
position(2) = parameters.Grid(2) - position(2) + 1;
y = position(2)*parameters.subBottom + (position(2)-1)*parameters.subTop + (position(2)-1)*parameters.size(2);

sub_position = [...
    parameters.figLeft   + x ...
    parameters.figBottom + y ...
    parameters.size(1) ...
    parameters.size(2)];


if nargin < 6 || isempty(cLIMs)
    clims(1) = min(Y2D(:));
    clims(2) = max(Y2D(:));
else
    clims = cLIMs;
end

if clims(1) > 0 & clims(1)<clims(2)*1e-13
    clims(1) = 0;
end

if clims(1) == clims(2)
    clims(2) = clims(1) + 1e-32;
end

axes('position',sub_position);

if nargin < 4
    imagesc(Y2D, clims)
    hold on
    if parameters.CountourLevel ~= 0
        [C,h] = contour(Y2D,parameters.CountourLevel*[1 1],'edgecolor','w', 'LineWidth',parameters.CountourLW);%,'LineWidth',4, 'ShowText','on');
    end
    
    set(gca,'YDir','normal','FontSize',parameters.fs)
    %axis  tight %off % equal tight off
    set(gca,'TickDir','out','TickLength',[0.01, 0.01])
else
    hold on
    if size(Y,1) == 1
        imagesc(X,Y,Y2D, clims)
    else
        yyaxis left
        imagesc(X,Y(1,:),Y2D, clims)
        ax = gca;
        ax.YColor = [0 0 0];
        % ylabel('Left Side')
        yyaxis right
        imagesc(X,Y(2,:),Y2D, clims)
        ax.YColor = 0*[1 0.3 0];
        %   ylabel('Right Side')
        Y = Y(2,:);
    end
    
    
    if parameters.CountourLevel ~= 0
        [C,h] = contour(X,Y,Y2D,parameters.CountourLevel*[1 1],'edgecolor','w', 'LineWidth',parameters.CountourLW);%,'LineWidth',4, 'ShowText','on');
    end
    
    set(gca,'YDir','normal','FontSize',parameters.fs)
    %axis tight% equal tight
    set(gca,'TickDir','out','TickLength',[0.01, 0.01])
    
    
end

colormap(get_cmap2(clims,1,1))

cb = colorbar();
if isfield(parameters,'colorbarVisible')
    cb.Visible = parameters.colorbarVisible;
end
box off
if isfield(parameters,'title_str')
    title(parameters.title_str)
    fh.Children(2).TitleFontSizeMultiplier = parameters.TitleFontSizeMultiplier;
end
fh.Children(1).Box = 'off';


fh.Children(1).TickDirection = 'out';

fh.Children(1).LineWidth = 2;
fh.Children(2).LineWidth = 2;
end


function cmap = get_cmap2(realY2D,midpoint,direction)
minY = min(realY2D(:));
maxY = max(realY2D(:));


if direction == 1
    cmap1 = flipud(bone);
    cmap2 = copper;
else
    cmap1 = flipud(copper);
    cmap2 = bone;
end

dminY = minY-midpoint;
dmaxY = maxY-midpoint;

if dminY*dmaxY < 0
    n = round(length(cmap1)*abs(dminY)/(abs(dminY)+abs(dmaxY)));
    cmap1 = interp1(1:length(cmap1),cmap1,linspace(1,length(cmap1),n));
    n = round(length(cmap2)*abs(dmaxY)/(abs(dminY)+abs(dmaxY)));
    cmap2 = interp1(1:length(cmap2),cmap2,linspace(1,length(cmap2),n));
    cmap = [cmap1; cmap2];
else
    cmap = [cmap2];
end
end
