% Calculate attenuation terms for various waveforms and background gradients (cylindrical geometry)
% Uses wfms_background_info.mat and makes attenuation_cylinder.mat file in the *model* folder.


clear all
close all

restoredefaultpath
addpath(genpath(fullfile(pwd,'add')));

waveform_dir = 'wfm_preclinical';
waveform_dir = fullfile(pwd,waveform_dir);

model_dir = fullfile(pwd,'model');
%mkdir(model_dir)

baseName = 'attenuation'; % D0 = 2.48e-9;

D0 = 2.48e-9; %2.48e-9; % 28 C (https://dtrx.de/od/diff/index.html#arrhenius)


addName = '_cylinder';
auxP.shape = 2; SaveAsName = [baseName addName];

R.Rs =  [1 2.5 5]*1e-6;

Parameters = [R.Rs; D0*ones(1,length(R.Rs))]';

% for rotating eigen spectra
load UDSRTriN15 % UDSRTriN300
auxP.cP = cos(UDSR.phi);
auxP.sP = sin(UDSR.phi);
auxP.cT = cos(UDSR.theta);
auxP.sT = sin(UDSR.theta);

load(fullfile(waveform_dir,'wfms_background_info.mat'),'wfms')

auxP.tortuosity = 0;

R.D0 = D0;
R.auxP = auxP;

M = numel(wfms);
N = size(Parameters,1);
Nbackground = size(wfms(1).PSb,1); % number of background gradient directions


for m = 1:M
    
    R.name{m} = wfms(m).name;
    
    for n = 1:N
        
        [l1 l2 l3] = fLw(Parameters(n,:),wfms(m),auxP,1);
        
        % desired terms
        R.att_d(m,n,:) = attenuation(wfms(m).PS_full,auxP,l1,l2,l3);
        
        for nbackground = 1:Nbackground
            % mixed terms desired and background
            PS = squeeze(wfms(m).PSdb(nbackground,:,:,:));
            R.att_db(m,n,nbackground,:) = attenuation(PS,auxP,l1,l2,l3);
            % background terms
            PS = squeeze(wfms(m).PSb(nbackground,:,:,:));
            R.att_b(m,n,nbackground,:) = attenuation(PS,auxP,l1,l2,l3);
        end
        
        
        disp(sprintf('wfm = %d/%d, R = %d/%d',m,M,n,N))
        
    end
    
end

save(fullfile(model_dir,[SaveAsName '.mat']),'R')


function att = attenuation(PS,auxP,l1,l2,l3)
att = zeros(length(auxP.cP),1); %auxP.Nwaveforms);


att = ...
    real(sum(l1.*PS(:,1,1)).*auxP.cP.^2.*auxP.cT.^2 + ...
    sum(l3.*PS(:,1,1)).*auxP.cP.^2.*auxP.sT.^2 + ...
    sum(l1.*PS(:,2,2)).*auxP.cP.^2 + ...
    sum(-2.*PS(:,1,3).*(l1 - l3)).*auxP.cP.*auxP.cT.*auxP.sT + ...
    sum(-2.*PS(:,1,2).*(l1 - l3)).*auxP.cP.*auxP.sP.*auxP.sT.^2 + ...
    sum(l1.*PS(:,2,2)).*auxP.cT.^2.*auxP.sP.^2 + ...
    sum(l3.*PS(:,3,3)).*auxP.cT.^2 + ...
    sum(-2.*PS(:,2,3).*(l1 - l3)).*auxP.cT.*auxP.sP.*auxP.sT + ...
    sum(l3.*PS(:,2,2)).*auxP.sP.^2.*auxP.sT.^2 + ...
    sum(l1.*PS(:,1,1)).*auxP.sP.^2 + ...
    sum(l1.*PS(:,3,3)).*auxP.sT.^2);
end