%Calculate and plot signals due to background gradients for IVIM as a function of b, Gb and direction.

clear all
close all

saveFig = 1;
fig_resolution = '-r300'; % dpi, e.g. '-r72' '-r600'

waveform_dir = 'wfm_preclinical';
load(fullfile(waveform_dir,'wfms_background_info.mat'),'wfms')


tunedLTE_ind = 1+4*[0 1 2]; % tuned
detunedLTE_ind = 3+4*[0 1 2]; % detuned
STE_ind = 4+4*[0 1 2];

col = [1 0 0; ...
    0 0 1; ...
    .4 .4 .4];


P.b = 1.5e9*linspace(0,1,300);
P.Gb = [30 10 3]*1e-3; % 30e-3 10e-3 3e-3 %Gb = 3e-3 in Szczepankiewicz F, Sjölund J: Cross-term-compensated gradient waveform design for tensor-valued diffusion MRI. J Magn Reson 2021; 328:106991.

P.Nb = length(P.b);
P.NGb = length(P.Gb);

% IVIM model: Ds =﻿1 mm2/ms, Df = 10 mm2/ms, ff = 0.1
Df = 10e-9;
Ds = 1e-9;
ff = 0.1;


lineStyle = '-';

showLabels = 0;

fig.fs = 16;
fig.lw = 3;


%lws = [8 6 3];
%lws = [4 3 2];
%lws = [4 2 1];
lws = 2*[1 1 1];

YLIM = [.1 1];

GR = 1/1.62;
fig.H = 400;
fig.W = fig.H*GR;

for M = 0:2
    P.traces(1,:,:) = wfms(tunedLTE_ind(1+M)).traces;
    P.traces(2,:,:) = wfms(detunedLTE_ind(1+M)).traces;
    P.traces(3,:,:) = wfms(STE_ind(1+M)).traces;
    
    for nGb = 1:length(P.Gb)
        Gb = P.Gb(nGb);
        for nwf = 1:size(P.traces,1)
            trb = P.traces(nwf,:,1);
            trbdb = P.traces(nwf,:,2);
            trbb = P.traces(nwf,:,3);
            
            for nb = 1:length(P.b)
                b = P.b(nb);
                bval = b*trb + sqrt(b)*Gb*trbdb + Gb^2*trbb;
                P.S(nwf,nb,:) = ff*exp(-bval*Df) + (1-ff)*exp(-bval*Ds);
            end
        end
        
        
        % ---------- show signal vs b ---------------------------------------------
        
        
        fh = figure(1);
        clf
        fh.Position = [400 100 fig.W fig.H];
        fh.Color = [1 1 1];
        
        X = P.b*1e-6;
        
        switch M %front-back
            case 0
                plot_order = [2 3 1];
            case 1
                plot_order = [1 3 2];
            case 2
                plot_order = [2 3 1];
        end
        
        hold on
        for nwf = plot_order
            Y = squeeze(P.S(nwf,:,:));
            plot(X,Y,'LineWidth',lws(nwf),'color',col(nwf,:),'LineStyle',lineStyle)
        end
        set(gca, 'YScale', 'log',...
            'FontSize',fig.fs,'box','off','TickDir','out','LineWidth',fig.lw,'TickLength',[.02 .02])
        % set(gca, 'YScale', 'lin',...
        %     'FontSize',fig.fs,'box','off','TickDir','out','LineWidth',fig.lw,'TickLength',[.02 .02])
        
        ylim(YLIM)
        if showLabels
            ylabel('signal')
            xlabel('b [s/mm^2]')
            title(sprintf('R = %.2f um',1e6*R.Rs(Nr)))
        end
        
        
        % lh = legend();
        % lh.Box = 'off';
        % lh.String = {'', '', '',''};
        
        
        if saveFig
            out_folder = 'add_test';
            mkdir(out_folder)
            fig_path = fullfile(out_folder,sprintf('test_background_IVIM_Svsb_M%d_Gb%d.png', M,round(Gb*1e3)));
            print(fh,fig_path,'-dpng',fig_resolution);
        end
        
        
    end
end
