%Analyze and plot b-tensor waveforms (time and frequency domain, b-tensor shape and q-trajectory).
%Makes the *_info.mat files in the waveform folders.
% see also: ﻿Lundell H, Lasič S. Diffusion Encoding with General Gradient Waveforms. In: Topgaard D, editor(s). Advanced Diffusion Encoding Methods in MRI: New Developments in NMR Volume 24. Cambridge, UK: Royal Society of Chemistry; 2020. p. 12–67

clear all
close all
restoredefaultpath
fig_offset = 0*1000;

% -------------------------- SELECT WAVEFORMS -------------------------------------------------------------
select_wfms = 1; %1 - preclinical, 2 - qMAS, 3 - clinical

switch select_wfms
    case 1 % preclinical
        
        waveform_dir = 'wfm_preclinical';
        waveform_names = {'M0_LTE_1','M0_LTE_2', 'M0_LTE_3', 'M0_STE', ...
            'M1_LTE_1','M1_LTE_2','M1_LTE_3','M1_STE',...
            'M2_LTE_1','M2_LTE_2','M2_LTE_3','M2_STE',...
            'PGSE_LTE'};
        
        dt = 20e-6;
        f0 = 300;
        
    case 2 % q-MAS
        waveform_dir = 'wfm_qMAS';
        waveform_names = {'qMAS_detunedLTE_SE_flip1','qMAS_SE_flip1','qMAS_tunedLTE_SE_flip1'};
        
        dt = 20e-6;
        f0 = 300;
        
    case 3 %clinical
        
        waveform_dir = 'wfm_clinical';
        waveform_names = {...
            'Trap__M0_N457_T46_maxG80_slew50_b526_b_eig1_1_1_Ax0_Ay0_Az0_g_M0', ...
            'Trap__M0_N571_T57_maxG54_slew34_b526_b_eig1_1_1_Ax0_Ay0_Az0_g_M0', ...
            'Trap__M1_N571_T57_maxG80_slew50_b526_b_eig1_1_1_Ax0_Ay0_Az0_g_M1', ...
            'Trap__M0_N671_T67_maxG42_slew26_b526_b_eig1_1_1_Ax0_Ay0_Az0_g_M0', ...
            'Trap__M1_N671_T67_maxG58_slew36_b525_b_eig1_1_1_Ax0_Ay0_Az0_g_M1', ...
            'Trap__M2_N671_T67_maxG80_slew50_b525_b_eig1_1_1_Ax0_Ay0_Az0_g_M2', ...
            'Trap_PGSE_LTE_b663_M0_N457_T46_maxG38_slew12_b526_b_eig1_0_0_Ax0_Ay0_Az0_g_M0', ...
            'Trap_PGSE_LTE_b663_M0_N571_T57_maxG27_slew7_b525_b_eig1_0_0_Ax0_Ay0_Az0_g_M0', ...
            'Trap_BP_LTE_b732_M1_N571_T57_maxG63_slew37_b525_b_eig1_0_0_Ax0_Ay0_Az0_g_M1',...
            'Trap_Welsh_LTE_b1086_M2_N571_T57_maxG73_slew55_b526_b_eig1_0_0_Ax0_Ay0_Az0_g_M2', ...
            'Trap_PGSE_LTE_b663_M0_N671_T67_maxG21_slew4_b525_b_eig1_0_0_Ax0_Ay0_Az0_g_M0', ...
            'Trap_BP_LTE_b732_M1_N671_T67_maxG47_slew24_b525_b_eig1_0_0_Ax0_Ay0_Az0_g_M1', ...
            'Trap_Welsh_LTE_b1086_M2_N671_T67_maxG57_slew35_b525_b_eig1_0_0_Ax0_Ay0_Az0_g_M2'};
        
        dt = 1e-4;
        f0 = 34.8/67.1*300;
end

fig_dir = fullfile(pwd,[waveform_dir '_fig'])
mkdir(fig_dir)


addpath(genpath(fullfile(pwd,'add')));


% ------   what to do  ---------
do.analyze = 1; % if 1 then calculate power spectra, if 0 then plot

do.save_figs = 0; % applies only if do.analyze = 0
fig_resolution = '-r300';%'-r600'; '-r300'; '-r72';

do.clear_info = 0; % applies only if do.analyze = 1, clear all old analysis, i.e. ..._info files

% some general parameters
do.show_SQ_tensors = 1; % superquadric glyphs, else peanuts
do.showBshape = 1;
do.show_trajectory = 1;


% ---- g(t), q(t), s(w) -----
do.plot_waveforms = 1; % scatter_plot_parameters.wfm
do.plot_full_spectra = 1; % scatter_plot_parameters.fullPS

% ---- b-tensor  -----
do.plot_color_weighted_spectra = 1; % col_spectra_parameters


% ------------ some general plotting parameters ------------------------------------------------------------

az = 60;
el = 40;

scatter_plot_parameters.wfm.plotLegends = 0;
scatter_plot_parameters.wfm.show_axes = 1;
scatter_plot_parameters.wfm.show_Yticks = 0;
scatter_plot_parameters.wfm.show_labels = 0;
scatter_plot_parameters.wfm.show_cum_trace = 1;
scatter_plot_parameters.wfm.normalizeXaxis = 0; % show time from 0 to 1
scatter_plot_parameters.wfm.units_g = 2; %1 - T/m, 2 - mT/m
scatter_plot_parameters.wfm.units_q = 3; %1 - 1/m, 2 - 1/mm, 3 - 1/um


% ---------------- PLOT COLOR CODED DIRECTIONAL SPECTRA PROJECTIONS -------------

col_spectra_parameters = get_col_spectra_parameters();
col_spectra_parameters.az = az;
col_spectra_parameters.el = el;

col_spectra_parameters.showSQ = do.show_SQ_tensors;
col_spectra_parameters.show_trajectory = do.show_trajectory;
col_spectra_parameters.tuning.colormap_showDw = 1; % show D(w) in the legend
col_spectra_parameters.showBshape = do.showBshape;

col_spectra_parameters.thresh = 0.998;
col_spectra_parameters.f0 = f0; %if >0 overrides thresh, if 0 then obtain from col_spectra_parameters.thresh

% ----------- FLOW ----------------------------
col_spectra_parameters.flow.do_shade = 1;

% ------------------------------------------------------------------------------------
% check which waveforms (and analysis) exist
d = dir(waveform_dir);
d = d(find(~strcmp({d.name},'.') & ~strcmp({d.name},'..') & ~strcmp({d.name},'.DS_Store')));
existing_waveform_names = {d(find(~contains({d.name},'_info'))).name};
d_info_files = d(find(contains({d.name},'_info')));

s = '{';
for n = 1:numel(existing_waveform_names)
    existing_waveform_names{n} = extractBefore(existing_waveform_names{n},'.mat');
    s = [s sprintf('''%s'',', existing_waveform_names{n})];
end
disp('list of all waveform names:')
disp([s(1:end-1) '}'])


% waveform info - interpolation
% if dt > 0 &  encoding_time = 0 then we set encoding_time = samples*dt (no interpolation)
% if dt = 0 &  encoding_time > 0 then we set dt = encoding_time/samples (no interpolation)
% if dt = 0 &  encoding_time = 0 then we interpolate samples = encoding_time/dt (do interpolation)

%dt = ...

encoding_time = 0;
set_encoding_power = 0; %2e9; % if 0 then use original from waveform else set new b

% FT info - for analysis
N0 = 1e7; % 0-padding

%rotate original waveform and calculated spectra - rotate vectors v(n,3) around ZYX
waveform_rotation_angles = 0*pi/4*[1 0 1];
% rotating spectra is slow, if all angles are 0, this rotation is skipped
PS_rotation_angle = waveform_rotation_angles;

%-------------------- PARAMETERS --------------------------------------

% ---------------- SCATTER PLOT PARAMETERS -------------
scatter_plot_parameters = get_scatter_plot_parameters_wfm(scatter_plot_parameters);
%scatter_plot_parameters.wfm.showPatch = 0;

% ---------------- SCATTER PLOT PARAMETERS: SPECTRA -------------
scatter_plot_parameters = get_scatter_plot_parameters_fullPS(scatter_plot_parameters);

scatter_plot_parameters.fullPS.thresh = 0.998;
scatter_plot_parameters.fullPS.f0 = f0; %if >0 overrides thresh, if 0 then obtain from scatter_plot_parameters.fullPS.thresh


% -----------------------------------------------------------------------
gmr = 26.75e7;

if do.analyze
    
    if do.clear_info
        for n = 1:numel(d_info_files)
            delete(fullfile(d_info_files(n).folder,d_info_files(n).name))
        end
    end
    
    for count = 1:numel(waveform_names)
        waveform_name = waveform_names{count};
        
        load(fullfile(waveform_dir,[waveform_name '.mat']))
        
        Nwave = size(g,1);
        
        if (encoding_time == 0) & (dt > 0)
            TE = Nwave*dt; %duration of the waveform without 0-padding
            t = linspace(0,TE,Nwave)';
        elseif (encoding_time > 0) & (dt == 0)
            dt = encoding_time/Nwave;
            TE = encoding_time; %duration of the waveform without 0-padding
            t = linspace(0,TE,Nwave)';
        else % do interpolation
            Nwave_new = round(encoding_time/dt);
            TE = Nwave_new*dt; %duration of the waveform without 0-padding
            
            %interpolate
            tmp = linspace(0,TE,Nwave)';
            t = linspace(0,TE,Nwave_new)';
            g = interp1(tmp,g,t);
            Nwave = Nwave_new;
        end
        
        
        [V, L, ~] = fB(gmr*cumsum(g)*dt,dt);
        b = L(1,1)+L(2,2)+L(3,3); % raw b
        
        if set_encoding_power ~= 0
            wfm.b = set_encoding_power;
            g = g*sqrt(set_encoding_power/b);
            [V, L, ~] = fB(gmr*cumsum(g)*dt,dt);
            
        else
            wfm.b = b;
        end
        
        % ------  encoding properties -----------------------
        slew_rate = diff(g)/dt;
        
        %Sjölund J, Szczepankiewicz F, Nilsson M, Topgaard D, Westin C-F, Knutsson H.
        %Constrained optimization of gradient waveforms for generalized diffusion encoding. J Magn Reson 2015; 261: 157–168.
        
        % Szczepankiewicz F, Westin CF, Nilsson M. Gradient waveform design for tensor-valued encoding in diffusion MRI. J Neurosci Methods 2020: 109007.
        wfm.heat = max(sum(g.^2)*dt); % also b/TE^2
        
        wfm.slew_rate_max = max(abs(slew_rate(:)));
        wfm.gradient_max = max(abs(g(:)));
        wfm.slew_x_gradient_max = wfm.slew_rate_max*wfm.gradient_max; % Power
        
        wfm.energy_max = wfm.gradient_max^2;
        
        wfm.heat_factor = wfm.heat/(wfm.gradient_max^2*TE); % unitless
        wfm.b_factor = wfm.b/(gmr^2*wfm.gradient_max^2*TE^3); % unitless (1/12 for constant gradient)
        wfm.b_per_T_per_maxG = wfm.b/(wfm.gradient_max*TE);
        wfm.TE = TE;
        
        
        % -----  Concomitant gradients -----------------------
        if (1)
            
            if ~exist('h180')
                h180 = -sign(t-TE/2); % dephasing direction (flips after 180)
            end
            
            %Szczepankiewicz F, Westin C-F, Nilsson M. Maxwell-compensated design of asymmetric gradient waveforms for tensor-valued diffusion encoding. Magn Reson Med 2019; 00: 1–14.
            %Figure 2 shows that N = 100 is a sufficient temporal resolution for the waveform optimization and that the optimization should constrain the Maxwell index to be less than ~1000 (mT/m)2ms.
            % To create additional headroom, we constrain all Maxwell‐compensated waveforms to have m ≤ 100 (mT/m)2ms.
            %Lasič S, Szczepankiewicz F, Dall’Armellina E, Das A, Kelly C, Plein S, et al. Motion compensated b-tensor encoding for in vivo cardiac diffusion-weighted imaging. NMR Biomed 2019: 1–16.
            
            % the actual gradient - not the "effective" one
            glab = g.*h180;
            
            gij = fouter2(glab);
            %Maxwell_ij = cumsum(gij.*h180)*dt; % proportional to Filip's M
            
            M = squeeze(sum(gij.*h180*dt));
            wfm.Maxwell_index =  sqrt(trace(M*M));
            wfm.norm_Maxwell_index = wfm.Maxwell_index/wfm.gradient_max^2;
            
            %Lasič S, Szczepankiewicz F, Dall’Armellina E, et al.: Motion compensated b-tensor encoding for in vivo cardiac diffusion-weighted imaging. NMR Biomed 2020; 33:e4213.
            % adapted from Eq. 13
            % 1-zz, 2-xz, 3-yz, 4-xx+yy
            Gxx = g(:,1).^2;
            Gyy = g(:,2).^2;
            Gzz = g(:,3).^2;
            Gxz = g(:,1).*g(:,3);
            Gyz = g(:,2).*g(:,3);
            Gc = [Gzz, -2*Gxz, -2*Gyz, 4*Gxx+4*Gyy]/4;
            Kc_perB0_t = gmr*cumsum(Gc.*h180)*dt;
            
            % due to numerical errors (e.g. small deviation of m0 from 0), it might be better to consider something like the K-terms at echo time vs. max. value
            
            wfm.Kc_perB0 = Kc_perB0_t(end,:);
            wfm.Kc_perB0_max = max(abs(Kc_perB0_t(:)));
            
            %             figure(1), clf, plot(t,Kc_perB0_t), return
            
            
            Gc = [g(:,3).^2, -2*g(:,1).*g(:,3), -2*g(:,2).*g(:,3), 4*g(:,1).^2+4*g(:,2).^2]/4;
            wfm.Kc_perB0 = gmr*sum(Gc.*h180)*dt;
        end
        
        if (1)
            % Szczepankiewicz F, Sjölund J: Cross-term-compensated gradient waveform design for tensor-valued diffusion MRI. J Magn Reson 2021; 328:106991.
            
            if ~exist('h180')
                h180 = -sign(t-TE/2); % dephasing direction (flips after 180)
            end
            H = gmr*cumsum(h180)*dt;
            q = gmr*cumsum(g)*dt;
            
            cross_term_sens_t = cumsum(q.*H)*dt;
            wfm.cross_term_sens = cross_term_sens_t(end,:);
            
            %figure(1), clf, plot(t,cross_term_sens_t), pause(.2)%, return
            
        end
        
        
        if (1) % exchange (for now just for XYZ waveforms and a mean projection on the sphere)
            %             Ning L, Nilsson M, Lasič S, Westin C-F, Rathi Y. Cumulant expansions for measuring water exchange using diffusion MRI. J Chem Phys 2018; 148
            %             Szczepankiewicz F, Westin CF, Nilsson M. Gradient waveform design for tensor-valued encoding in diffusion MRI. J Neurosci Methods 2020: 109007.
            %             q4(t) = integral_0^tau q^2(x)q^2(x+t) dx
            %             Gamma = 2/b^2*integral_0^tau q4(t)*t*dt
            
            
            q = gmr*cumsum(g)*dt;
            
            [~, L, ~] = fB(q,dt);
            b = trace(L);
            
            wfm.exchange_Gamma = Exchange_Gamma(q*[1 0 0; 0 1 0; 0 0 1]',b,t)
            
        end
        
        clear h180 % for batch processing
        
        [wfm.bt, wfm.bT, wfm.f_full, wfm.f, wfm.PS_full, wfm.PS, wfm.cPS, wfm.trPS, wfm.g, wfm.q] = ...
            fWaveform_PS_EnsureB(g, b, N0, TE);
        
        T = fTensorShape(wfm.bT);
        
        wfm.b_delta = T.Delta;
        wfm.b_eta = T.Eta;
        wfm.b_FA = T.FA;
        wfm.V = T.V; % after normalization to b = 1
        wfm.l = T.l; % after normalization to b = 1
        wfm.trace = T.m; % after normalization to b = 1
        
        save(fullfile(waveform_dir,[waveform_name '_info.mat']),'wfm')
        clear g wfm
    end
end

if ~do.analyze
    for count = 1:numel(waveform_names)
        close all
        waveform_name = waveform_names{count}
        
        if ~exist('waveform_full_names')
            waveform_full_names = waveform_names;
        end
        if length(waveform_full_names) ~= length(waveform_names)
            waveform_full_names = waveform_names;
        end
        
        waveform_full_name = waveform_full_names{count};
        
        
        load(fullfile(waveform_dir,[waveform_name '_info.mat']),'wfm')
        
        gx = wfm.g(:,1);
        gy = wfm.g(:,2);
        gz = wfm.g(:,3);
        
        qx = wfm.q(:,1);
        qy = wfm.q(:,2);
        qz = wfm.q(:,3);
        
        bt = wfm.bt;
        PS_full = wfm.PS_full;
        PS = wfm.PS;
        cPS = wfm.cPS;
        trPS = wfm.trPS;
        
        
        f_full = wfm.f_full;
        f = wfm.f;
        dt = wfm.TE/length(wfm.g);
        
        load UDSRTriN1000
        u = [UDSR.x UDSR.y UDSR.z];
        
        if do.save_figs
            mkdir(fullfile(fig_dir, waveform_full_name))
        end
        
        
        
        %% plotting
        count_fig_offset = 1000; %reserve 1-1000 for invisible figs
        count_figs = count_fig_offset;
        
        % --------------------------- plot just time-domain waveforms  -----------------------------
        if do.plot_waveforms
            
            % --------------------------- gradient waveform  -----------------------------
            count_figs = count_figs+1;
            
            fig_h = figure(count_figs+fig_offset);
            
            clf
            ax1 = axes('Position',[scatter_plot_parameters.wfm.left_margin ...
                scatter_plot_parameters.wfm.bottom_margin+2*scatter_plot_parameters.wfm.height+2*scatter_plot_parameters.wfm.yspacing1+scatter_plot_parameters.wfm.yspacing2 ...
                scatter_plot_parameters.wfm.width scatter_plot_parameters.wfm.height]);
            hold on
            
            X = linspace(0,1,length(gx))';
            if ~scatter_plot_parameters.wfm.normalizeXaxis
                X = X*dt*length(gx)*1e3;
            end
            Y = scatter_plot_parameters.wfm.scale_g*[gx gy gz];
            %rotate vectors v(n,3) around ZYX
            Y = fVectorRot(Y, waveform_rotation_angles(1), waveform_rotation_angles(2), waveform_rotation_angles(3));
            
            nonzero_ind = sum(abs(Y))/sum(abs(Y(:))) > 0.01;% find zero elements
            Y = Y(:,nonzero_ind);
            col = scatter_plot_parameters.wfm.colors(:,nonzero_ind);
            
            legendText1{:,3} = scatter_plot_parameters.wfm.legendText(nonzero_ind);
            legendText1{:,2} = scatter_plot_parameters.wfm.legendText(nonzero_ind);
            legendText1{:,1} = 'PS';
            
            % plot g(t)
            plot_scatter_axis(X, Y, [], scatter_plot_parameters.wfm.g_label, col, '-', scatter_plot_parameters.wfm)
            
            % --------------------------- dephasing waveform  -----------------------------
            
            ax2 = axes('Position',[scatter_plot_parameters.wfm.left_margin scatter_plot_parameters.wfm.bottom_margin+scatter_plot_parameters.wfm.height+scatter_plot_parameters.wfm.yspacing1+scatter_plot_parameters.wfm.yspacing2 scatter_plot_parameters.wfm.width scatter_plot_parameters.wfm.height]);
            hold on
            
            Y = scatter_plot_parameters.wfm.scale_q*[qx qy qz];
            %rotate vectors v(n,3) around ZYX
            Y = fVectorRot(Y, waveform_rotation_angles(1), waveform_rotation_angles(2), waveform_rotation_angles(3));
            
            nonzero_ind = sum(abs(Y))/sum(abs(Y(:))) > 0.01;% find zero elements
            Y = Y(:,nonzero_ind);
            col = scatter_plot_parameters.wfm.colors(:,nonzero_ind);
            
            % plot q(t)
            plot_scatter_axis(X, Y, scatter_plot_parameters.wfm.t_label, scatter_plot_parameters.wfm.q_label, col, '-', ...
                scatter_plot_parameters.wfm)
            
            
            ax3 = axes('Position',[scatter_plot_parameters.wfm.left_margin scatter_plot_parameters.wfm.bottom_margin+0*scatter_plot_parameters.wfm.yspacing1 scatter_plot_parameters.wfm.width scatter_plot_parameters.wfm.height]);
            
            hold on
            % --------------------------- mean power spectrum  -----------------------------
            
            
            if scatter_plot_parameters.fullPS.f0 == 0
                f0 = f(max(find(trPS<scatter_plot_parameters.fullPS.thresh)));
            else
                f0 = scatter_plot_parameters.fullPS.f0;
            end
            
            ind = find(f_full>=-f0 & f_full<=f0);
            
            X = f_full(ind);
            Y = PS_full(ind,1,1) + PS_full(ind,2,2) + PS_full(ind,3,3);
            col = 0*[1 1 1]';
            
            % plot PS
            plot_scatter_axis(X,Y,scatter_plot_parameters.wfm.f_label,scatter_plot_parameters.wfm.trPS_label, col, '-', ...
                scatter_plot_parameters.wfm)
            
            if scatter_plot_parameters.wfm.show_cum_trace
                maxY = max(Y);
                Y = trPS(f<f0);
                Y = [flipud(Y); Y(2:end)];
                X = f(f<f0);
                X = [-fliplr(X) X(2:end)];
                hold on
                plot(X,Y/max(Y)*maxY,':k','LineWidth',scatter_plot_parameters.wfm.lwIn)
            end
            
            
            if do.save_figs
                figName = [waveform_full_name '_waveforms' ]; % _' num2str(tensor_evolution_parameters.az,3) num2str(tensor_evolution_parameters.el,3)];
                figName = [figName '.png'];
                print(fig_h,fullfile(fig_dir, waveform_full_name, figName),'-dpng',fig_resolution);
            end
            
            % ------   show legend separately -------
            if scatter_plot_parameters.wfm.plotLegends
                
                if scatter_plot_parameters.wfm.showPatch
                    legendType = 'patch';
                else
                    legendType = 'line';
                end
                h = fig_h; %findobj(gca);
                
                count_figs = count_figs+1;
                fig_h = figure(count_figs+fig_offset);
                clf
                h = copyobj(h.Children,fig_h);
                for n = 1:numel(h)
                    h(n).XLim(1) = 0; % to avoid clashing with legend
                    h(n).XLim(2) = h(n).XLim(2)*2; % to avoid clashing with legend
                    
                    h_obj = findobj(h(n),'Type',legendType);
                    lh = legend(flipud(h_obj),legendText1{:,n});
                    lh.Location = 'best';
                    lh.Box = 'off';
                end
                
                if do.save_figs
                    figName = [waveform_full_name  '_waveform_legend' ]; % _'  num2str(tensor_evolution_parameters.az,3) num2str(tensor_evolution_parameters.el,3)];
                    figName = [figName '.png'];
                    print(fig_h,fullfile(fig_dir, waveform_full_name, figName),'-dpng',fig_resolution);
                    
                end
            end
            
        end
        
        
        
        % --------------------------- full cross correlation spectra  -----------------------------
        if do.plot_full_spectra
            
            if scatter_plot_parameters.fullPS.f0 == 0
                f0 = f(max(find(trPS<scatter_plot_parameters.fullPS.thresh)));
            else
                f0 = scatter_plot_parameters.fullPS.f0;
            end
            scatter_plot_parameters.fullPS_new = scatter_plot_parameters.fullPS;
            scatter_plot_parameters.fullPS_new.f0 = f0;
            
            
            ind = find(f_full>=-f0 & f_full<=f0);
            
            X = f_full(ind);
            
            rotatedPS = fRotateTensor(PS_full(ind,:,:),PS_rotation_angle);
            
            
            Y = [rotatedPS(:,1,1) rotatedPS(:,2,2) rotatedPS(:,3,3) ...
                rotatedPS(:,1,2) rotatedPS(:,2,3) rotatedPS(:,1,3)];
            
            realY = real(Y);
            imagY = imag(Y);
            
            real_ind = sum(abs(realY))/sum(abs(realY(:))) > 0.01;% find zero elements
            imag_ind = sum(abs(imagY))/sum(abs(imagY(:))) > 0.01;% find zero elements
            
            Y = [realY(:,real_ind) imagY(:,imag_ind)];
            
            for n = 1:6
                real_LineStyle{n} = scatter_plot_parameters.fullPS.real_LineStyle;
                imag_LineStyle{n} = scatter_plot_parameters.fullPS.imag_LineStyle;
            end
            
            LineStyles = [real_LineStyle(real_ind) imag_LineStyle(imag_ind)];
            col = [scatter_plot_parameters.fullPS.colors(:,real_ind) scatter_plot_parameters.fullPS.colors(:,imag_ind)];
            legendText1 = {'','','','','',''};
            %legendText1 = [scatter_plot_parameters.fullPS.legendText(real_ind) scatter_plot_parameters.fullPS.legendText(imag_ind)];
            
            count_figs = count_figs+1;
            hplotFullPSf = figure(count_figs+fig_offset);
            
            axes('Position',[scatter_plot_parameters.fullPS.left_margin scatter_plot_parameters.fullPS.bottom_margin ...
                scatter_plot_parameters.fullPS.width scatter_plot_parameters.fullPS.height])
            
            hold on
            
            % plot full PS
            plot_scatter_axis(X,Y,scatter_plot_parameters.fullPS.f_label,scatter_plot_parameters.fullPS.PS_label, col, '-', scatter_plot_parameters.fullPS)
            %plot_scatter_axis(X,Y,scatter_plot_parameters.fullPS.f_label,scatter_plot_parameters.fullPS.PS_label, col, LineStyles, scatter_plot_parameters.fullPS)
            
            axis tight
            %pbaspect([1.618 1 1])
            
            if ~isempty(scatter_plot_parameters.fullPS.frq_ticks)
                xticks(scatter_plot_parameters.fullPS.frq_ticks)
            end
            
            set(gca,'LineWidth',scatter_plot_parameters.fullPS.lwOut,'Box','off','TickDir','out','TickLength',[.02 .02],'FontSize',...
                scatter_plot_parameters.fullPS.fs)
            if ~scatter_plot_parameters.fullPS.show_axes
                axis off
            elseif ~scatter_plot_parameters.fullPS.show_Yticks
                set(gca,'ytick',[])
            end
            %hplotFullPSf.Visible = 'off';
            
            if scatter_plot_parameters.fullPS.showPatch
                legendType = 'patch';
            else
                legendType = 'line';
            end
            
            h = findobj(gca,'Type',legendType);
            
            if (scatter_plot_parameters.fullPS.plotLegends)
                % show with legend separately
                h = gca; %findobj(gca);
                
                count_figs = count_figs+1;
                hplotfullPSf_legend = figure(count_figs+fig_offset);
                clf
                h = copyobj(h,hplotfullPSf_legend);
                h.XLim = h.XLim*2; % to avoid clashing with legend
                hplotfullPSf_legend.Visible = 'on';
                if (0)
                    h = findobj(gca); % make them invisible
                    for count = 1:numel(h)
                        if strcmp(h(count).Type,'line') || strcmp(h(count).Type,'patch')
                            h(count).Visible = 'off';
                        end
                    end
                end
                h = findobj(gca,'Type',legendType);
                
                lh = legend(flipud(h),legendText1);
                lh.Location = 'best';
                legend boxoff
                lh.ItemTokenSize(1) = lh.ItemTokenSize(1)*scatter_plot_parameters.fullPS.legend_width*scatter_plot_parameters.fullPS.lwOut;
                
            end
            
            
            
            if do.save_figs
                figName = [waveform_full_name  '_fullPS' ]; % _' num2str(tensor_evolution_parameters.az,3) num2str(tensor_evolution_parameters.el,3)];
                figName = [figName '.png'];
                print(hplotFullPSf,fullfile(fig_dir, waveform_full_name, figName),'-dpng',fig_resolution);
                
                if exist('hplotfullPSf_legend')
                    figName = [waveform_full_name  '_fullPS_legend' ]; % _' num2str(tensor_evolution_parameters.az,3) num2str(tensor_evolution_parameters.el,3)];
                    figName = [figName '.png'];
                    print(hplotfullPSf_legend,fullfile(fig_dir, waveform_full_name, figName),'-dpng',fig_resolution);
                end
            end
            
            
        end
        
        % --------------------------- power spectra  -----------------------------
        
        
        %rotate tesnors around ZYX
        rotatedPS = fRotateTensor(PS,PS_rotation_angle);
        %rotate vectors v(n,3) around ZYX
        rotated_g = fVectorRot([gx gy gz], PS_rotation_angle(1), PS_rotation_angle(2), PS_rotation_angle(3));
        rotated_q = fVectorRot([qx qy qz], PS_rotation_angle(1), PS_rotation_angle(2), PS_rotation_angle(3));
        
        
        
        % --------------------------- color wighted -------------------------------
        if do.plot_color_weighted_spectra
            
            
            count_figs = count_figs+1;
            
            bt_plot = bt(end,:,:);
            %rotate tensor around ZYX
            bt_plot = fRotateTensor(bt_plot,waveform_rotation_angles);
            
            [hplot, hlegend, col_spectra_parameters_new] = plotColorWeightedSpectraExtended(count_figs+fig_offset, count_figs+1+fig_offset, f, rotatedPS, trPS, UDSR, rotated_g, rotated_q, bt_plot, dt, col_spectra_parameters);
            
            count_figs = count_figs+1;
            
            
            if do.save_figs
                figName = [waveform_full_name  '_RGB_Power_colormap'  ];
                figName = [figName '.png'];
                print(hlegend,fullfile(fig_dir, waveform_full_name, figName),'-dpng',fig_resolution);
            end
            
            if do.save_figs
                figName = [waveform_full_name  '_RGB_Power' ];
                figName = [figName '.png'];
                print(hplot,fullfile(fig_dir, waveform_full_name, figName),'-dpng',fig_resolution);
            end
            
            
        end
        
        
        if do.save_figs
            
            info_path = fullfile(fig_dir, waveform_full_name,'fig_info.mat');
            if isfile(info_path) % load pre-existing fields
                load(fullfile(fig_dir, waveform_full_name,'fig_info.mat'),'fig_info');
            end
            
            fig_info.waveform_rotation_angles = waveform_rotation_angles;
            fig_info.PS_rotation_angle = PS_rotation_angle;
            
            
            
            if do.plot_full_spectra
                fig_info.scatter_plot_parameters.fullPS = scatter_plot_parameters.fullPS_new;
            end
            
            
            % save also waveform parameters to display
            % remove some heavy data
            wfm = rmfield(wfm,'bt');
            wfm = rmfield(wfm,'f_full');
            wfm = rmfield(wfm,'f');
            wfm = rmfield(wfm,'PS_full');
            wfm = rmfield(wfm,'PS');
            wfm = rmfield(wfm,'cPS');
            wfm = rmfield(wfm,'trPS');
            wfm = rmfield(wfm,'g');
            wfm = rmfield(wfm,'q');
            
            fig_info.wfm = wfm;
            
            save(fullfile(fig_dir, waveform_full_name,'fig_info.mat'),'fig_info');
            
        end
        
        
    end
end


%--------------------------------- FUNCTIONS ------------------------------

function [f PS] = fPS(F,dt,mode)
%  Power Spectra of F

N = size(F,2);
NT = size(F,1);

f = 1/dt*(0:(NT/2))/NT;

if (exist('mode', 'var'))
    if strcmp(mode,'positive')
        mode = 1;
        PS = zeros(length(f),N,N);
    end
else
    mode = 0;
end

if ~mode
    f = [-fliplr(f(2:end)) f(1:end)];
    PS = zeros(NT+1,N,N);
end


for m = 1:N
    for n = 1:N
        
        ps = fft(F(:,m)).*conj(fft(F(:,n)));
        
        if mode
            PStmp = ps(1:length(f));
        else
            PStmp = fftshift(ps);
            PStmp = [PStmp; PStmp(1)];
        end
        
        %         figure(1),clf
        %         plot(f,real(PStmp),f,imag(PStmp))
        %         sum(real(PStmp))
        %         sum(imag(PStmp))
        
        PS(:,m,n) = PStmp;
        
    end
end

end


function [px,py] = patchXY(x,y1,y2,n)
l = length(x);
j = 1;
for i = 1:n:(l-n)
    
    px(1,j) = x(i);
    py(1,j) = y1(i);
    
    px(2,j) = x(i);
    py(2,j) = y2(i);
    
    px(3,j) = x(i+n);
    py(3,j) = y2(i+n);
    
    px(4,j) = x(i+n);
    py(4,j) = y1(i+n);
    
    j = j + 1;
end

end


function Tu = fTensorProjection(T,u)
% T(ind1,ind2,ind3)
% ind1 = consecutive tensor to project time or frequency
% ind2,ind3 = T indices
% u(n,i) are n directions with i=1...3 components

if (numel(size(T)) == 2) % make it order 3
    t(1,:,:) = T(:,:);
    T = t;
end

for ind1 = 1:size(T,1)
    for n = 1:size(u,1)
        Tu(n,ind1) = 0;
        for ind2 = 1:3
            for ind3 = 1:3
                Tu(n,ind1) = Tu(n,ind1)+T(ind1,ind2,ind3)*u(n,ind2)*u(n,ind3);
            end
        end
    end
end

Tu = squeeze(Tu);
end


function rotT = fRotateTensor(T,ang,opt)
% size(T) = [n,3,3], where n are different tensors, it can handle size(T) = [3,3]
% use opt.type for XYZ or Euler
% use opt.dir for forward or backward direction

if ~prod(ang == [0 0 0]) % check if all angles are not 0
    
    if (nargin < 3)
        opt = {};
    end
    
    if (~isfield(opt, 'type'))
        opt.type = 1;
    end
    
    if (~isfield(opt, 'dir'))
        opt.dir = 1;
    end
    
    if opt.dir == 2
        ang = -ang;
    end
    
    
    switch opt.type
        case 1 % XYZ
            phiX = ang(1);
            phiY = ang(2);
            phiZ = ang(3);
            
            R1 = [ % Rx
                1 0 0
                0 cos(phiX) -sin(phiX)
                0 sin(phiX) cos(phiX)];
            
            R2 = [ % Ry
                cos(phiY) 0 sin(phiY)
                0 1 0
                -sin(phiY) 0 cos(phiY)];
            
            R3 = [ % Rz
                cos(phiZ) -sin(phiZ) 0
                sin(phiZ) cos(phiZ) 0
                0 0 1];
            
        case 2 % Euler
            alpha = ang(1);
            beta = ang(2);
            gamma = ang(3);
            
            R1 = [ % Rgamma
                cos(gamma) sin(gamma) 0
                -sin(gamma) cos(gamma) 0
                0 0 1];
            
            R2 = [ % Rbeta
                cos(beta) 0 -sin(beta)
                0 1 0
                sin(beta) 0 cos(beta)];
            
            R3 = [ % Ralpha
                cos(alpha) sin(alpha) 0
                -sin(alpha) cos(alpha) 0
                0 0 1];
    end
    
    switch opt.dir
        case 1 % foreward
            R = R3*R2*R1;
        case 2 % backward
            R = R1*R2*R3;
    end
    
    trR = R';
    
    
    if (numel(size(T)) == 2) % make it rank 3
        t(1,:,:) = T(:,:);
        T = t;
    end
    
    
    %rotT = RTR'
    for n = 1:size(T,1)
        t = squeeze(T(n,:,:));
        rotT(n,:,:) = R*t*trR;
    end
    
else
    rotT = T;
end

rotT = squeeze(rotT);


end


function vout = fVectorRot(v, Ax, Ay, Az)
%rotate vectors v(n,3) around ZYX

v1 = v(:,1);
v2 = v(:,2);
v3 = v(:,3);

sAx = sin(Ax);
cAx = cos(Ax);
sAy = sin(Ay);
cAy = cos(Ay);
sAz = sin(Az);
cAz = cos(Az);

vout(:,1) = conj(v3).*(sAx.*sAz + cAx.*cAz.*sAy) - conj(v2).*(cAx.*sAz - cAz.*sAx.*sAy) + cAy.*cAz.*conj(v1);
vout(:,2) = conj(v2).*(cAx.*cAz + sAx.*sAy.*sAz) - conj(v3).*(cAz.*sAx - cAx.*sAy.*sAz) + cAy.*sAz.*conj(v1);
vout(:,3) = cAy.*sAx.*conj(v2) - sAy.*conj(v1) + cAx.*cAy.*conj(v3);

end


function [V, L, B] = fB(q, dt)
B = q'*q*dt;
[V, L] = eig(B);
end


function [bt, bT, f_full, f, PS_full, PS, cPS, trPS, gmat, qmat] = fWaveform_PS_EnsureB(gmat, b, N0, TE)
% ensure b value in time
% ensure b = 1 in frequency domain
% subscript _full refers to positive and negative frequencies
gmr = 26.75e7;


N = length(gmat);
t = linspace(0,TE,N);
dt = mean(diff(t));

qmat = gmr*cumsum(gmat)*dt;

for nWave = 1:size(gmat,3)
    
    g = squeeze(gmat(:,:,nWave));
    q = squeeze(qmat(:,:,nWave));
    
    [V, L, ~] = fB(q,dt);
    
    g = g/sqrt(trace(L))*sqrt(b);
    q = q/sqrt(trace(L))*sqrt(b);
    
    
    for n = 1:length(q)
        for i = 1:3
            for j = 1:3
                qq(n,i,j) = q(n,i)*q(n,j)*dt;
            end
        end
    end
    
    bT(:,:,nWave) = sum(qq);
    bt(:,:,:,nWave) = cumsum(qq);
    
    q_zeropad = [zeros(N0,3); q; zeros(N0,3)];
    
    
    %[f PS] = fPS(q_zeropad,dt,'positive');
    [f_full, PS_full] = fPS(q_zeropad,dt);
    
    PS_all(:,:,:,nWave) = PS_full;
    
    gmat(:,:,nWave) = g;
    qmat(:,:,nWave) = q;
    
    % find high frequency index (cumulative power up to 1-1e-6)
    ind = find(f_full>=0);
    f = f_full(ind);
    PS = PS_full(ind,:,:);
    P = cumsum(PS(:,1,1) + PS(:,2,2) + PS(:,3,3))/(sum(PS(:,1,1) + PS(:,2,2) + PS(:,3,3)));
    ind = max(find(P<1-1e-6));
    maxP(nWave) = P(ind); % close to 1
    f0(nWave) = f(ind);
    
end

% eliminate some super high frequency points
f0 = max(f0);

ind = find(f_full>=-f0 & f_full<=f0);
PS_full = PS_all(ind,:,:,:);
f_full = f_full(ind);

ind = find(f_full>=0);
f = f_full(ind);
PS = PS_full(ind,:,:,:);

% ensure b = 1
for nWave = 1:size(gmat,3)
    trB = trace(real(squeeze(sum(PS(:,:,:,nWave))))); % total power
    
    PS(:,:,:,nWave) = maxP(nWave)*PS(:,:,:,nWave)/trB;
    
    cPS(:,:,:,nWave) = cumsum(PS(:,:,:,nWave));
    trPS(:,nWave) = cPS(:,1,1,nWave) + cPS(:,2,2,nWave) + cPS(:,3,3,nWave);
    
    trB = trace(real(squeeze(sum(PS_full(:,:,:,nWave))))); % total power
    PS_full(:,:,:,nWave) = maxP(nWave)*PS_full(:,:,:,nWave)/trB;
    
end
end

function [X1, Y1] = interpolatePlot(X,Y,N)
if N > 0
    X1 = linspace(min(X),max(X),N);
    Y1 = interp1(X,Y,X1);
else
    X1 = X;
    Y1 = Y;
end
end

function Tensor = fTensorShape(Ts_in)
% size(Ts_in) = 3 x 3 or n x 3 x 3

if numel(size(Ts_in)) == 2
    Ts(1,:,:) = Ts_in;
else
    Ts = Ts_in;
end

for n = 1:size(Ts,1)
    T = squeeze(Ts(n,:,:));
    [V,L] = eig(T);
    m = trace(T)/3;
    l = [L(1,1) L(2,2) L(3,3)];
    [l, index] = sort(l,'descend');
    V = V(:,index);
    
    if m == 0
        tmp = l-1; % to fix the m = 0 singularity
    else
        tmp = l/m-1;
    end
    [~, index] = sort(abs(tmp),'descend');
    Delta = (tmp(index(1)) - (tmp(index(2))+tmp(index(3)))/2)/3;
    tmp = tmp(index(2))-tmp(index(3));
    if abs(tmp) < 1e-9
        Eta = 0;
    else
        Eta = abs(tmp/2/Delta);
    end
    
    Tensor(n).Delta = round(100*Delta)/100;
    Tensor(n).Eta = round(100*Eta)/100;
    Tensor(n).FA = sqrt(3/2*sum((l-m).^2)/sum(l.^2));
    
    Tensor(n).V = V;
    Tensor(n).l = l;
    Tensor(n).m = m;
    
end
end

function makeCylindricalAxes(L, r, n_circ, n_seg, r_seg, col)
% L - length of axes
% r - axes width
% n_circ - number of points on the circle
% n_seg - number of segments (divisions of line)
% r_seg - ratio length/spacing of segments
% color (3x3)

if nargin < 6
    col = eye(3);
end

L_spacing = L/n_seg;
L_seg = r_seg*L_spacing;
spacing = linspace(-L+L_seg/2,L-L_seg/2,2*n_seg)+L_seg;

[Xx, Yx, Zx, Xy, Yy, Zy, Xz, Yz, Zz] = makeCylindricalAxes1(L_seg,r,n_circ);



for i = 1:2:length(spacing)
    Xx_tmp = Xx+spacing(i);
    sx(1,i) = fill3(Xx_tmp(1,:),Yx(1,:),Zx(1,:),col(1,:),'LineStyle','none');
    sx(2,i) = fill3(Xx_tmp(2,:),Yx(2,:),Zx(2,:),col(1,:),'LineStyle','none');
    sx(3,i) = surf(Xx_tmp,Yx,Zx,'facecolor',col(1,:),'LineStyle','none');
    
    Yy_tmp = Xx+spacing(i);
    sy(1,i) = fill3(Xy(1,:),Yy_tmp(1,:),Zy(1,:),col(2,:),'LineStyle','none');
    sy(2,i) = fill3(Xy(2,:),Yy_tmp(2,:),Zy(2,:),col(2,:),'LineStyle','none');
    sy(3,i) = surf(Xy,Yy_tmp,Zy,'facecolor',col(2,:),'LineStyle','none');
    
    Zz_tmp = Xx+spacing(i);
    sz(1,i) = fill3(Xz(1,:),Yz(1,:),Zz_tmp(1,:),col(3,:),'LineStyle','none');
    sz(2,i) = fill3(Xz(2,:),Yz(2,:),Zz_tmp(2,:),col(3,:),'LineStyle','none');
    sz(3,i) = surf(Xz,Yz,Zz_tmp,'facecolor',col(3,:),'LineStyle','none');
end




    function [Xx, Yx, Zx, Xy, Yy, Zy, Xz, Yz, Zz] = makeCylindricalAxes1(L1,r,n)
        
        [Xz,Yz,Zz] = cylinder(r,n);
        
        Zz = 2*L1*(Zz-0.5);
        
        Rx = [1 0 0; 0 0 -1; 0 1 0];
        Ry = [0 0 1; 0 1 0; -1 0 0];
        
        
        XYZ1 = Ry*[Xz(1,:); Yz(1,:); Zz(1,:)];
        XYZ2 = Ry*[Xz(2,:); Yz(2,:); Zz(2,:)];
        Xx(1,:) = XYZ1(1,:); Yx(1,:) = XYZ1(2,:); Zx(1,:) = XYZ1(3,:);
        Xx(2,:) = XYZ2(1,:); Yx(2,:) = XYZ2(2,:); Zx(2,:) = XYZ2(3,:);
        
        XYZ1 = Rx*[Xz(1,:); Yz(1,:); Zz(1,:)];
        XYZ2 = Rx*[Xz(2,:); Yz(2,:); Zz(2,:)];
        Xy(1,:) = XYZ1(1,:); Yy(1,:) = XYZ1(2,:); Zy(1,:) = XYZ1(3,:);
        Xy(2,:) = XYZ2(1,:); Yy(2,:) = XYZ2(2,:); Zy(2,:) = XYZ2(3,:);
        
        
    end
end

function gij = fouter2(g)
tim = size(g,1);
dir = size(g,2);

gij = zeros(tim,dir,dir);

for i = 1:dir
    for j = 1:dir
        gij(:,i,j) = g(:,i).*g(:,j);
    end
end

end

function exGamma = Exchange_Gamma(qu,b,t)
%Ning L, Nilsson M, Lasic S, Westin C-F, Rathi Y. Cumulant expansions for measuring water exchange using diffusion MRI. J Chem Phys 2018; 148
% calculate Gamma for all projections qu (Nt x Ndirection)
% q4(t) = integral_0^tau q^2(x)q^2(x+t) dx
% Gamma = 2*integral_0^tau q4(t)*t*dt
% q = gmr*cumtrapz(g)*dt;

% ensure t and qu are column vectors [qu can have several colums]
if size(t,1) < size(t,2)
    t = t';
end

if size(qu,1) < size(qu,2)
    qu = qu';
end

q2 = qu.^2;
dt = t(2)-t(1);

N = length(qu);

for n = 1:N
    q4(n,:) = sum(q2(1:N-n+1,:).*q2(n:end,:))*dt;
end

exGamma = 2/b^2*sum(q4.*t)*dt;

end

function mat = fmap(mat,min_val,max_val)
mat = (mat-min(mat(:)))/(max(mat(:))-min(mat(:)));
mat = min_val + (max_val - min_val)*mat;
end

function val = expShape(val, plato, par)
% exponential shape (plato, band and p as parameters)
% plato = 0 __/\__ or plato = 1 --\/--

band = par.band;
p = par.p;
amp = par.amp;

if band > 0
    val = amp*exp(-1/band*val.^p);
else
    val = ones(size(val));
end

if plato == 1
    val = 1-val;
end
end

function [hplot, hlegend, parameters] = plotColorWeightedSpectraExtended(figureN, legendN, f_in, PS_in, trPS_in, ODF, g, q, bt, dt, parameters)

% cutoff for color coded spectra (after tuning)
if parameters.f0 == 0
    f0 = max(f_in(trPS_in<parameters.thresh));
else
    f0 = parameters.f0;
end

% limit frq range
ind = find(f_in<=f0);
f = f_in(ind);
PS = PS_in(ind,:,:);


% ------- interpolate to speed up
[f, PS] = interpolatePS(f,PS, 1000);
tracePS = PS(:,1,1)+PS(:,2,2)+PS(:,3,3);


% ----------- PS projections and normalization --------
u = [ODF.x ODF.y ODF.z];
PSu = real(fTensorProjection(PS,u));

% normalize to trace
PSu = PSu/sum(tracePS); % mean(sum(PSu,2)) = 1/3
tracePS = tracePS/sum(tracePS);
% figure(1),clf, hold on, plot(tracePS,'--b')

showSQ = parameters.showSQ & norm(eye(3)-3*bt/trace(bt))>0.1; % ensure that bt is not spherical



if parameters.showBshape == 1
    
    if showSQ
        ODF = fSuperQuadTensor(bt,50,3,parameters.SQoffset,[]);
        
        u = ODF.verts./ODF.Pdir';
        PSu = real(fTensorProjection(PS,u));
        % normalize to trace
        PSu = PSu/sum(tracePS); % mean(sum(PSu,2)) = 1/3
    else
        ODF.Pdir = fTensorProjection(bt,[ODF.x ODF.y ODF.z]);
        ODF.verts = repmat(ODF.Pdir,[1 3]).*[sin(ODF.theta).*cos(ODF.phi)...
            sin(ODF.theta).*sin(ODF.phi) ...
            cos(ODF.theta)];
    end
    
else
    showSQ = 0;
    ODF.Pdir = ones(ODF.N,1); %*trace(bt);
    ODF.verts = repmat(ODF.Pdir,[1 3]).*[sin(ODF.theta).*cos(ODF.phi)...
        sin(ODF.theta).*sin(ODF.phi) ...
        cos(ODF.theta)];
end

% cumulative trace
cum_tracePS = cumsum(tracePS);

% normalize ODF
maxPdir = max(ODF.Pdir);
ODF.verts = ODF.verts/maxPdir;
ODF.Pdir = ODF.Pdir/maxPdir;

% ----------- D(w) for color map ----------------------------
if parameters.colormap.D2Rm4>0
    if parameters.colormap.restricted
        % restricted
        D0 = parameters.colormap.D0;
        R = (D0^2/parameters.colormap.D2Rm4)^(1/4);
        Dw = DwSpherical(2*pi*f,R,D0,0,50)';
    else
        % incoherent flow (%FT of exponential autocorrelation exp(-|t|/tau) -> 2a/(a^2+w^2), a = 1/tau, a = 1/1e-1)
        D0 = parameters.colormap.D0;
        tau = (1/parameters.colormap.D2Rm4)^(1/2);  %tau = 0.001;
        a = 1./tau;
        Dw = 2*a./(a^2+(2*pi*f').^2);
        Dw = D0*Dw/max(Dw);
    end
end
%     figure(1),clf
%     plot(Dw)

Ncolors = size(PS,1);

if isempty(parameters.binning)
    if parameters.colormap.D2Rm4>0
        % split color according to Dw
        if parameters.colormap.restricted
            Nred = max(find(Dw/D0<1/3));
            Ngreen = max(find(Dw/D0<2/3))-Nred;
            Nblue = Ncolors-Nred-Ngreen;
        else
            Nred = max(find(Dw/D0>2/3));
            Ngreen = max(find(Dw/D0>1/3))-Nred;
            Nblue = Ncolors-Nred-Ngreen;
        end
        
        binning(1) = Nred/Ncolors;
        binning(2) = Ngreen/Ncolors;
        parameters.split_color_weighted_spectra = 'according to even binnin of diffusion spectrum';
    else
        % split color according to power
        Nred = max(find(cum_tracePS<1/3));
        Ngreen = max(find(cum_tracePS<2/3))-Nred;
        Nblue = Ncolors-Nred-Ngreen;
        binning(1) = Nred/Ncolors;
        binning(2) = Ngreen/Ncolors;
        parameters.split_color_weighted_spectra = 'according to even binnin of encoding spectral trace';
    end
else
    % split bins
    Nred = round(Ncolors*parameters.binning(1));
    Ngreen = round(Ncolors*parameters.binning(2));
    Nblue = Ncolors-Nred-Ngreen;
    %binning = [];
    parameters.split_color_weighted_spectra = 'according to custom bins';
end
display(['binning = ' num2str(binning)])
display(['f0 = ' num2str(f0)])

parameters.binning = binning;
parameters.f0 = f0;


% red-green-blue
col(:,1) = [1*ones(1,Nred) 0*ones(1,Ngreen) 0*ones(1,Nblue)];
col(:,2) = [0*ones(1,Nred) 1*ones(1,Ngreen) 0*ones(1,Nblue)];
col(:,3) = [0*ones(1,Nred) 0*ones(1,Ngreen) 1*ones(1,Nblue)];


hlegend = figure(legendN);
clf

patch('Faces',[1:4],'Vertices',...
    [0 0; Nred 0; Nred 1; 0 1],...
    'EdgeColor','none','FaceColor',parameters.path_brightness*[1 0 0],'FaceAlpha',1);
patch('Faces',[1:4],'Vertices',...
    [Nred 0; Nred+Ngreen 0; ...
    Nred+Ngreen 1; Nred 1],...
    'EdgeColor','none','FaceColor',parameters.path_brightness*[0 1 0],'FaceAlpha',1);
patch('Faces',[1:4],'Vertices',...
    [Nred+Ngreen 0; Ncolors 0; ...
    Ncolors 1; Nred+Ngreen 1],...
    'EdgeColor','none','FaceColor',parameters.path_brightness*[0 0 1],'FaceAlpha',1);


if parameters.colormap.D2Rm4>0
    plot_function = Dw'/D0; %tracePS/max(tracePS).*Dw'/D0;
else
    plot_function = cum_tracePS;
end


hold on
plot(1:length(plot_function),plot_function,'k-','Linewidth',6)
plot(1:length(tracePS),tracePS/max(tracePS),'k--','Linewidth',4)


box off
pbaspect([5 1 1])
axis off


% ------------------------- COLOR CODE -----------------------
% normalize each direction
PSu_norm = PSu./sum(PSu,2);
for n = 1:size(PSu,1)
    C(n,:) = sum(repmat(PSu_norm(n,:),3,1)'.*col);
end

% shade flow sensitivity
if parameters.flow.do_shade
    
    flow_sens = PSu_norm(:,1)./mean(PSu_norm,2);
    flow_sens(isnan(flow_sens)) = max(flow_sens);
    flow_sens = fmap(flow_sens,0,1);
    
    mix = 1-expShape(flow_sens, 0, parameters.flow.shade); %__/\__
    C = repmat(1-mix,1,3).*C ...
        + repmat(mix,1,3).*repmat(parameters.flow.shade.color,length(C),1);
end

hplot = figure(figureN);
clf


pa = patch('Faces',ODF.tri,'Vertices',ODF.verts/max(ODF.Pdir));
axis tight off, axis square, axis equal


if (1)
    hold on
    L = 1.5;
    %     plot3(L*[-1:1],0*[-1:1],0*[-1:1],'--','Color',0.4*[1 0 0],'Linewidth',1*parameters.ax_width)
    %     plot3(0*[-1:1],L*[-1:1],0*[-1:1],'--','Color',0.4*[0 0.8 0],'Linewidth',1*parameters.ax_width)
    %     plot3(0*[-1:1],0*[-1:1],L*[-1:1],'--','Color',0.4*[0 0 1],'Linewidth',1*parameters.ax_width)
    makeCylindricalAxes(L,parameters.ax_width*L/200,20,parameters.ax_style(1),parameters.ax_style(2))%,col)
end

if (parameters.show_trajectory) % make q trajectory
    hold on
    factor = 1.2;
    tmp = q;
    tmp = parameters.trajectory_scale*tmp/max(abs(tmp(:)));
    plot3(tmp(:,1),tmp(:,2),tmp(:,3),'-k','Linewidth',parameters.trajectory_lw)
end

set(pa,'FaceColor','interp','FaceVertexCData',C,...
    'EdgeColor',parameters.edge_brightness*[1 1 1],'LineWidth',.5,'EdgeAlpha',parameters.edge_alpha,'FaceAlpha',parameters.face_alpha)

set(pa,'FaceLighting','phong',...%'FaceColor','interp',...
    'AmbientStrength',parameters.ambient,'SpecularStrength',parameters.specular)


set(gca,'XTick',[],'YTick',[],'ZTick',[],'FontSize',14)


if ~isempty(parameters.lightPos1)
    light('Position',parameters.lightPos1,'Style','infinite'), lighting gouraud
end

if ~isempty(parameters.lightPos2)
    light('Position',parameters.lightPos2,'Style','infinite'), lighting gouraud
end

h = gca;
h.View = [parameters.az,parameters.el];
h.CameraViewAngle = parameters.CameraViewAngle;

end

function [f_interp, PS_interp] = interpolatePS(f,PS,Ninterp)
Ninterp = 2*round(Ninterp/2)+1;
for i = 1:3
    for j = 1:3
        PS_interp(:,i,j) = interp1(1:length(PS),PS(:,i,j),linspace(1,length(PS),Ninterp));
    end
end
%PS = PS_interp;
%tracePS = interp1(1:length(tracePS),tracePS,linspace(1,length(tracePS),Ninterp)');
f_interp = interp1(1:length(f),f,linspace(1,length(f),Ninterp));
end

function plot_scatter_axis(X,Y,x_label,y_label,col,line_styles,parameters)
% ----   patches between 0 and Y   -------

for n = 1:size(Y,2)
    if numel(line_styles) == 1
        line_style = char(line_styles);
    else
        line_style = line_styles{n};
    end
    
    if (parameters.showPatch)
        [px,py] = patchXY(X,0*Y(:,n),Y(:,n),parameters.Npatch);
        p = patch(px,py,'k');
        p.FaceColor = col(:,n);
        p.FaceAlpha = parameters.patch_face_alpha;
        p.EdgeColor = 'none';
        p.LineStyle = line_style;
        
    end
    [X1,Y1] = interpolatePlot(X,Y(:,n),parameters.Ninterpolate);
    plot(X1,Y1,line_style,'Color',col(:,n),'LineWidth',parameters.lwIn,'MarkerSize',10)
end

axis tight
%pbaspect([1.62*ar 1 1])
set(gca,'LineWidth',parameters.lwOut,'Box','off','TickDir','out','TickLength',[.02 .02],'FontSize',parameters.fs)
if ~parameters.show_axes
    axis off
elseif ~parameters.show_Yticks
    set(gca,'ytick',[])
end

if parameters.show_labels
    xlabel(x_label);
    ylabel(y_label);
end
end

function scatter_plot_parameters = get_scatter_plot_parameters_wfm(scatter_plot_parameters)
% what is shown in the legend, lines or patches?  (only non-zero components are shown)
scatter_plot_parameters.wfm.showPatch = 1;

if scatter_plot_parameters.wfm.normalizeXaxis
    scatter_plot_parameters.wfm.t_label = 'time [x 1/\tau]';
else
    scatter_plot_parameters.wfm.t_label = 'time [ms]';
end

switch scatter_plot_parameters.wfm.units_g
    case 1
        scatter_plot_parameters.wfm.scale_g = 1;
        scatter_plot_parameters.wfm.g_label = '\bfg\rm [T/m]';
    case 2
        scatter_plot_parameters.wfm.scale_g = 1e3;
        scatter_plot_parameters.wfm.g_label = '\bfg\rm [mT/m]';
end

switch scatter_plot_parameters.wfm.units_q
    case 1
        scatter_plot_parameters.wfm.scale_q = 1;
        scatter_plot_parameters.wfm.q_label = '\bfq\rm [1/m]';
    case 2
        scatter_plot_parameters.wfm.scale_q = 1e-3;
        scatter_plot_parameters.wfm.q_label = '\bfq\rm [1/mm]';
    case 3
        scatter_plot_parameters.wfm.scale_q = 1e-6;
        scatter_plot_parameters.wfm.q_label = '\bfq\rm [1/\mum]';
end

scatter_plot_parameters.wfm.trPS_label = 'trace(\bfs\rm)';
scatter_plot_parameters.wfm.f_label = 'frequency [Hz]';
scatter_plot_parameters.wfm.PS_label = 'power spectra';

scatter_plot_parameters.wfm.colors(:,1) = 1.0*[1 0 0];%XX
scatter_plot_parameters.wfm.colors(:,2) = 0.8*[0 1 0];%YY
scatter_plot_parameters.wfm.colors(:,3) = 1.0*[0 0 1];%ZZ
scatter_plot_parameters.wfm.legendText = {'X','Y','Z'};

scatter_plot_parameters.wfm.Npatch = 1; % avery Np-th point is used (sometimes this can speed up plotting)

scatter_plot_parameters.wfm.Ninterpolate = 1e5; %1e5; % if > 0 interpolate all scatter plots
scatter_plot_parameters.wfm.lwIn = 4;
scatter_plot_parameters.wfm.lwOut = 4;
scatter_plot_parameters.wfm.fs = 18;
scatter_plot_parameters.wfm.patch_face_alpha = 0.5;

scatter_plot_parameters.wfm.aspectRatio = 1.618*4;%1.618;


if scatter_plot_parameters.wfm.show_axes
    scatter_plot_parameters.wfm.left_margin = 0.09;
    scatter_plot_parameters.wfm.bottom_margin = 0.1; %0.12;
    scatter_plot_parameters.wfm.width = 0.88;
    scatter_plot_parameters.wfm.height = 0.19;
    scatter_plot_parameters.wfm.yspacing1 = 0.11; % 0.07; % top
    scatter_plot_parameters.wfm.yspacing2 = 0.07; % 0.07; % bottom
    
    if scatter_plot_parameters.wfm.show_labels
        scatter_plot_parameters.wfm.left_margin = 0.14;
        scatter_plot_parameters.wfm.bottom_margin = 0.15; %0.12;
        scatter_plot_parameters.wfm.width = 0.82;
        scatter_plot_parameters.wfm.height = 0.18;
        scatter_plot_parameters.wfm.yspacing1 = 0.11;% 0.07; % top
        scatter_plot_parameters.wfm.yspacing2 = 0.07; %0.07; % bottom
    end
    
else
    scatter_plot_parameters.wfm.left_margin = 0.025;
    scatter_plot_parameters.wfm.bottom_margin = 0.05;
    scatter_plot_parameters.wfm.width = 0.95;
    scatter_plot_parameters.wfm.height = 0.25;
    scatter_plot_parameters.wfm.yspacing1 = 0.05;
    scatter_plot_parameters.wfm.yspacing2 = 0.00;
end

% -----------------------------

scatter_plot_parameters.wfm.remove_axis_g = 0;
scatter_plot_parameters.wfm.remove_axis_q = 0;
scatter_plot_parameters.wfm.spaceY_g = 0;
scatter_plot_parameters.wfm.spaceX_g = 0;
scatter_plot_parameters.wfm.spaceY_q = 0;
scatter_plot_parameters.wfm.spaceX_q = 0;

if scatter_plot_parameters.wfm.show_axes
    scatter_plot_parameters.wfm.scale_size_g = 1.14;
    scatter_plot_parameters.wfm.left_margin_g = -0.82;
    scatter_plot_parameters.wfm.bottom_margin_g = -0.85;
    
    scatter_plot_parameters.wfm.scale_size_q = scatter_plot_parameters.wfm.scale_size_g;
    scatter_plot_parameters.wfm.left_margin_q = scatter_plot_parameters.wfm.left_margin_g;
    scatter_plot_parameters.wfm.bottom_margin_q = -1.18;
    
    if scatter_plot_parameters.wfm.show_labels
        scatter_plot_parameters.wfm.scale_size_g = 1.1;
        scatter_plot_parameters.wfm.left_margin_g = -0.72;
        scatter_plot_parameters.wfm.bottom_margin_g = -0.767;
        
        scatter_plot_parameters.wfm.scale_size_q = scatter_plot_parameters.wfm.scale_size_g;
        scatter_plot_parameters.wfm.left_margin_q = scatter_plot_parameters.wfm.left_margin_g;
        scatter_plot_parameters.wfm.bottom_margin_q = -1.07;
    end
    
else
    scatter_plot_parameters.wfm.scale_size_g = 1.25;
    scatter_plot_parameters.wfm.left_margin_g = -0.95;
    scatter_plot_parameters.wfm.bottom_margin_g = -0.95;
    
    scatter_plot_parameters.wfm.scale_size_q = scatter_plot_parameters.wfm.scale_size_g;
    scatter_plot_parameters.wfm.left_margin_q = scatter_plot_parameters.wfm.left_margin_g;
    scatter_plot_parameters.wfm.bottom_margin_q = -1.25;
end

end

function scatter_plot_parameters = get_scatter_plot_parameters_fullPS(scatter_plot_parameters);

scatter_plot_parameters.fullPS = scatter_plot_parameters.wfm;
%scatter_plot_parameters.fullPS.legendType = 'line'; %'patch' 'line'; % other
scatter_plot_parameters.fullPS.lwIn = 6;

scatter_plot_parameters.fullPS.legendText = {'XX','YY','ZZ','XY','YZ','XZ'};

scatter_plot_parameters.fullPS.colors(:,1) = 1.0*[1 0 0];%XX
scatter_plot_parameters.fullPS.colors(:,2) = 0.8*[0 1 0];%YY
scatter_plot_parameters.fullPS.colors(:,3) = 1.0*[0 0 1];%ZZ
scatter_plot_parameters.fullPS.colors(:,4) = 0.8*[1 1 0];%XY
scatter_plot_parameters.fullPS.colors(:,5) = 0.8*[0 1 1];%YZ
scatter_plot_parameters.fullPS.colors(:,6) = 0.8*[1 0 1];%XZ

scatter_plot_parameters.fullPS.showPatch = 0;
scatter_plot_parameters.fullPS.thresh = 0.995;
scatter_plot_parameters.fullPS.f0 = 0; %if >0 overrides thresh, if 0 then obtain from col_spectra_parameters.thresh

scatter_plot_parameters.fullPS.real_LineStyle = '-';
scatter_plot_parameters.fullPS.imag_LineStyle = ':';
scatter_plot_parameters.fullPS.legend_width = 0.18;
scatter_plot_parameters.fullPS.frq_ticks = []; % frequency axis

if scatter_plot_parameters.fullPS.show_axes
    scatter_plot_parameters.fullPS.left_margin = 0.09;
    scatter_plot_parameters.fullPS.width = scatter_plot_parameters.wfm.width;
    scatter_plot_parameters.fullPS.height = scatter_plot_parameters.fullPS.width/1.618;
    scatter_plot_parameters.fullPS.bottom_margin = 0.15;
else
    scatter_plot_parameters.fullPS.left_margin = 0.02;
    scatter_plot_parameters.fullPS.width = 1-2*scatter_plot_parameters.fullPS.left_margin;
    scatter_plot_parameters.fullPS.height = scatter_plot_parameters.fullPS.width/1.618;
    scatter_plot_parameters.fullPS.bottom_margin = 0.05;
end
end

function col_spectra_parameters = get_col_spectra_parameters();

col_spectra_parameters.showBshape = 1;
col_spectra_parameters.showSQ = 0;
col_spectra_parameters.SQoffset = 0.18;

col_spectra_parameters.thresh = 0.98;
col_spectra_parameters.f0 = 0; %if >0 overrides thresh, if 0 then obtain from col_spectra_parameters.thresh
col_spectra_parameters.binning = [];


% bins 1
% ----------- D(w) for color map ----------------------------
col_spectra_parameters.colormap.restricted = 1; % if 0 then mimic incoherent flow
col_spectra_parameters.colormap.D0 = 2e-9;
R = 5e-6;
col_spectra_parameters.colormap.D2Rm4 = col_spectra_parameters.colormap.D0^2/R^4;  %D^2/R^4, if 0 then bin according to encoding power (lower value for encoding power at lower frequencies)


% ----------- FLOW ----------------------------
col_spectra_parameters.flow.do_shade = 1;
col_spectra_parameters.flow.shade.band = 0.5; % lower value for wider band
col_spectra_parameters.flow.shade.amp = 1;
col_spectra_parameters.flow.shade.p = 18;
col_spectra_parameters.flow.shade.color = 0.0*[1 1 0];

col_spectra_parameters.show_trajectory = 1;
col_spectra_parameters.trajectory_scale = 1.15;
col_spectra_parameters.trajectory_lw = 4;
col_spectra_parameters.ax_style = [20,0.5]; % number of segments, ratio length/spacing of segments (0-1: 0.5 is even)
col_spectra_parameters.ax_width = 2;

col_spectra_parameters.path_brightness = 1; %0.8; for showing scale
col_spectra_parameters.edge_brightness = 0.5; %0.8;
col_spectra_parameters.edge_alpha = 0;%0.5
col_spectra_parameters.face_alpha = 0.88;
col_spectra_parameters.ambient = 0.95;
col_spectra_parameters.specular = 0.3;

col_spectra_parameters.az = 60; col_spectra_parameters.el = 40;
col_spectra_parameters.lightPos1 =  1*[1 -0.5 .5];
col_spectra_parameters.lightPos2 = 0*[0 1 0.5];

col_spectra_parameters.CameraViewAngle = 6; % zoom

end



